@@ -7,7 +7,7 @@ use File::BaseDir qw/xdg_data_files/;
 use File::MimeInfo::Magic;
 use File::MimeInfo::Applications;
 
-our $VERSION = '0.17';
+our $VERSION = '0.20';
 
 our $STYLE = 'auto'; # either "auto" "menu" or "plain"
 
@@ -64,7 +64,7 @@ sub import {
 	$STYLE = $opts{style} if defined $opts{style};
 }
 
-=item C<populate_menu(MENU, FILE, MIMETYPE)>
+=item C<populate_menu(MENU, FILE, MIMETYPE, STYLE)>
 
 Convenience method that adds all applictions that can open this file
 to the menu.
@@ -73,10 +73,13 @@ MENU should be of class L<Gtk2::Menu>.
 MIMETYPE is optional and FILE can either be a filename or an array reference
 with filenames.
 
+STYLE can be used to override the default style.
+
 =cut
 
 sub populate_menu {
-	my ($class, $menu, $file, $mimetype) = @_;
+	my ($class, $menu, $file, $mimetype, $style) = @_;
+	$style ||= $STYLE;
 	
 	my @items = $class->menuitems($file, $mimetype);
 
@@ -88,8 +91,8 @@ sub populate_menu {
 	}
 
 	if (
-		($STYLE eq 'menu' and @items > 1) or
-		($STYLE eq 'auto' and @items > 4)
+		($style eq 'menu' and @items > 1) or
+		($style eq 'auto' and @items > 4)
 	) { # use a submenu
 		my $item = Gtk2::MenuItem->new('Open _With ...');
 		$menu->prepend($item);
