@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.19';
+our $VERSION = '0.25';
 
 use strict;
 use Gtk2;
@@ -74,15 +74,21 @@ Simple constructor.
 sub INIT_INSTANCE {
 	my $self = shift;
 	$self->{edit_mode_tags} = [];
+	$self->{_edit_mode_string} = '';
 	$self->{undo} = [];
 	$self->{redo} = [];
 	$self->signal_connect(mark_set => sub {
 			my ($self, $iter, $mark) = @_;
+			my $name = $mark->get_name;
 			$self->set_edit_mode_from_cursor()
-				if $mark->get_name eq 'insert';
+				if defined($name) and $name eq 'insert';
 		} );
 	$self->signal_connect_after(insert_text => \&on_insert);
 	$self->signal_connect(delete_range => \&on_delete_range);
+	$self->signal_connect(apply_tag => \&on_apply_tag);
+	$self->signal_connect(remove_tag => \&on_remove_tag);
+	$self->signal_connect(begin_user_action => sub {$$self{_ua} = 1});
+	$self->signal_connect(end_user_action => sub {$$self{_ua} = 0});
 }
 
 =item C<clear()>
@@ -172,19 +178,18 @@ Like C<insert_blocks()> but inserts at current cursor position.
 
 =cut
 
-my $_Verbatim = 0;
-
 sub insert_blocks_at_cursor {
 	# Make sure not to modify content of the reference !
 	# TODO: check default editable / region editable
 	my $self = shift;
-	
-	$self->signal_handlers_block_by_func(\&on_insert);
+	$self->signal_handlers_block_by_func($_)
+		for \&on_insert, \&on_apply_tag, \&on_remove_tag;
 	my $begin = $self->get_iter_at_mark( $self->get_insert )->get_offset;
 	$self->_insert_blocks(@_);
 	my $end = $self->get_iter_at_mark( $self->get_insert )->get_offset;
 	$self->_record_step('I', $begin, $end, @_);
-	$self->signal_handlers_unblock_by_func(\&on_insert);
+	$self->signal_handlers_unblock_by_func($_)
+		for \&on_insert, \&on_apply_tag, \&on_remove_tag;
 	return 1;
 }
 
@@ -193,18 +198,23 @@ sub _insert_blocks {
 
 	for (@blocks) {
 		unless (ref $_) {
+			my $text = $_;
+			$self->insert_at_cursor($text);
 			if (@{$self->{edit_mode_tags}}) {
-				my $iter = $self->get_iter_at_mark( $self->get_insert );
-				$self->insert_with_tags(
-					$iter, $_, @{$self->{edit_mode_tags}} );
+				my $end = $self->get_iter_at_mark(
+					$self->get_insert );
+				my $start = $end->copy;
+				$start->backward_chars(length $text);
+				$self->remove_all_tags($start, $end);
+				$self->apply_tag($_, $start, $end)
+					for @{$self->{edit_mode_tags}};
 			}
-			else { $self->insert_at_cursor($_) }
 		}
 		else {
 			my ($type, $meta) = @$_[0, 1];
 			my $tag;
 			if ($type eq 'image') {
-				$self->insert_image_from_file(@$meta{qw/src width height/});
+				$self->insert_image_from_file(%$meta);
 				next;
 			}
 			elsif ($type eq 'link') {
@@ -214,18 +224,13 @@ sub _insert_blocks {
 			else { $tag = $self->get_tag_table->lookup($type) }
 			
 			if ($tag) {
-				$_Verbatim++ if $type eq 'Verbatim'; # FIXME
-				push @{$self->{edit_mode_tags}}, $tag;
+				#push @{$self->{edit_mode_tags}}, $tag;
+				@{$self->{edit_mode_tags}} = ($tag);
 				$self->_insert_blocks(@$_[2 .. $#$_]); # recurs
-				pop @{$self->{edit_mode_tags}};
-				$_Verbatim-- if $type eq 'Verbatim'; # FIXME
+				#pop @{$self->{edit_mode_tags}};
+				@{$self->{edit_mode_tags}} = ();
 			}
 			elsif ($type eq 'Page' or $type eq 'Para') {
-				if ($$meta{base}) {
-					$self->{base} = $$meta{base};
-					$self->{base} =~ s#^file://##;
-					#warn "base set to: $self->{base}\n";
-				}
 				$self->_insert_blocks(@$_[2 .. $#$_]); # recurs
 			}
 			else { die "Unknown tag in parse tree: $type" }
@@ -235,47 +240,41 @@ sub _insert_blocks {
 			$self->insert_at_cursor("\n"x$lines) if $lines;
 		}
 	}
-
 }
 
-=item C<insert_image_from_file(FILENAME, WIDTH, HEIGHT, ITER)>
+=item C<insert_image_from_file(%PARAM)>
 
-Insert an image in the buffer at the cursor
-or at ITER if ITER is specified.
+Insert an image in the buffer at the cursor. The params should at least contain
+'file' and optionally 'src', 'width' and 'height'.
 
-WIDTH and HEIGHT are optional arguments. If both are set the image
-is resized to these dimensions. If one of both is zero or undefined
-the image is scaled with fixed ratio.
+The 'file' param is used as source for the image. If both 'width' and 'height'
+are set the image is resized to these dimensions. If one of both is zero or
+undefined the image is scaled with fixed ratio. All other params are attached
+to the image as data field.
 
 Use C<insert_blocks()> if you want this action to be un-doable. 
 
 =cut
 
 sub insert_image_from_file {
-	my ($self, $src, $w, $h, $iter) = @_;
-	($w, $iter) = (undef, $w) if @_ == 3 and ref $w;
-	my $file = $self->{base} ? File::Spec->rel2abs($src, $self->{base}) : $src;
-	#warn "file: $src, base: $self->{base} => $file\n";
+	my ($self, %param) = @_;
+	die "BUG: image without 'file' attribute" unless length $param{file};
 
 	my $pixbuf = undef;
-	if (-f $file and -r _) {
-		unless ($w || $h) {
-			eval { $pixbuf =
-				Gtk2::Gdk::Pixbuf->new_from_file($file) }
-		}
-		else {
-			eval { $pixbuf =
-				Gtk2::Gdk::Pixbuf->new_from_file_at_size(
-					$file, $w || -1, $h || -1) }
-		}
-	}
-	$pixbuf ||= Gtk2::Image->new->render_icon('gtk-missing-image', 'dialog')->copy;
-	die "Could not insert image: $file\n" unless $pixbuf;
+	my ($w, $h) = @param{'width', 'height'};
+	eval {
+		$pixbuf = ($w || $h)
+			? Gtk2::Gdk::Pixbuf->new_from_file_at_size(
+				$param{file}, $w || -1, $h || -1 )
+			: Gtk2::Gdk::Pixbuf->new_from_file(
+				$param{file} ) ;
+	} if -f $param{file} and -r _ ;
+	$pixbuf ||= Gtk2::Image->new->render_icon(
+			'gtk-missing-image', 'dialog' )->copy;
+	die "Could not insert image: $param{file}\n" unless $pixbuf;
 	
-	$pixbuf->{image_src} = $src;
-	$pixbuf->{image_file} = $file;
-	$pixbuf->{resized} = ($w || $h) ? [$w, $h] : 0;
-	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	$pixbuf->{image_data} = \%param;
+	my $iter ||= $self->get_iter_at_mark( $self->get_insert );
 
 	#my $mark = $self->create_mark(undef, $iter, 0);
 	#$mark->set_visible(1);
@@ -325,9 +324,7 @@ sub get_parse_tree {
 				$image->forward_chars(length $1);
 				my $pixbuf = $image->get_pixbuf;
 				if ($pixbuf) {
-					my $meta = { src => $pixbuf->{image_src} };
-					@$meta{qw/width height/} = @{$pixbuf->{resized}}
-						if $pixbuf->{resized};
+					my $meta = $pixbuf->{image_data};
 					push @{$stack[-1]}, ['image', $meta];
 				}
 				$image->forward_char;
@@ -399,7 +396,7 @@ sub get_parse_tree {
 	# Filter out empty tags
 	@$root = grep {
 		defined($_) and
-		(ref($_) and ($$_[0] !~ /^(link|image)/)) 
+		(ref($_) and ($$_[0] ne 'image')) 
 			? (@$_ > 2)
 			: length($_)
 	} @$root;
@@ -642,12 +639,15 @@ sub toggle_edit_mode_tag {
 
 sub on_insert {
 	my ($self, $end, $string) = @_;
+
 	# length in argument list does not understand utf8
 	my $start = $end->copy;
 	$start->backward_chars(length $string);
+
+	$self->_record_step('I', $start->get_offset, $end->get_offset, $string);
+		# record this step _before_ triggering any steps from tags
 	$self->remove_all_tags($start, $end);
 	$self->apply_tag($_, $start, $end) for @{$self->{edit_mode_tags}};
-	$self->_record_step('I', $start->get_offset, $end->get_offset, $string);
 }
 
 sub on_delete_range {
@@ -656,30 +656,38 @@ sub on_delete_range {
 	$self->_record_step('D', $begin->get_offset, $end->get_offset, $tree);
 }
 
+sub on_apply_tag {
+	my ($self, $tag, $begin, $end) = @_;
+	my $name = $tag->get_property('name') || '';
+	return if $name =~ /^gtkspell/;
+	$self->_record_step('A', $begin->get_offset, $end->get_offset, $tag);
+}
+
+sub on_remove_tag {
+	my ($self, $tag, $begin, $end) = @_;
+	my $name = $tag->get_property('name') || '';
+	return if $name =~ /^gtkspell/;
+	$self->_record_step('R', $begin->get_offset, $end->get_offset, $tag);
+}
+
 =item C<undo>
 
-TODO
+Undo one editing step.
 
 =item C<redo>
 
-TODO
+Redo one editing step.
 
 =cut
 
-my %reverse_step = (
-	I => 'D',
-	D => 'I',
-	A => 'R',
-	R => 'A',
-);
+my %_reverse_step = qw/I D D I A R R A/; # two pairs of complementary actions
 
 sub undo {
 	my $self = shift;
 	return 0 unless @{$self->{undo}};
 	my $step = pop @{$self->{undo}};
 	push @{$self->{redo}}, [@$step];
-	$$step[0] = $reverse_step{$$step[0]};
-	$self->_do_step(@$step);
+	$self->_do_step($step, 1);
 	return 1;
 }
 
@@ -688,32 +696,68 @@ sub redo {
 	return 0 unless @{$self->{redo}};
 	my $step = pop @{$self->{redo}};
 	push @{$self->{undo}}, [@$step];
-	$self->_do_step(@$step);
+	$self->_do_step($step, 0);
 	return 1;
 }
 
 sub _record_step {
 	my ($self, $step, $begin, $end, @data) = @_;
-	return if $self->{_undo_lock};
-	$self->{_redo} = [];
+	#return unless $$self{_ua} and !$$self{_undo_lock};
+	return if $$self{_undo_lock};
+	#warn "# Record ($$self{_ua}): $step, $begin, $end, @data\n";
+	$self->_fold_steps if @{$self->{redo}};
 	push @{$self->{undo}}, [$step, $begin, $end, @data];
 }
 
+sub _fold_steps {
+	# Triggered if you start editing after an undo
+	# "folds" the undo sequence instead of throwing it away
+	my $self = shift;
+	my @steps = reverse @{$self->{redo}}; # reverse because we pop in undo()
+
+	# redo the whole stack
+	$self->{redo} = [];
+	push @{$self->{undo}}, @steps;
+
+	# add the "undo" action as undoable action
+	@steps = reverse map [@$_], @steps; # copy, no ref, reverse again
+	$$_[0] = $_reverse_step{$$_[0]} for @steps;
+	push @{$self->{undo}}, [@steps]; # push whole group as single step
+}
+
 sub _do_step {
-	my ($self, $step, $begin, $end, @data) = @_;
+	# does one step or one group of steps
+	my ($self, $ref, $reverse) = @_;
+	my @steps = (ref($$ref[0]) ? @$ref : $ref);
+	@steps = reverse @steps if $reverse;
+	
 	$self->{_undo_lock} = 1;
-	$_ = $self->get_iter_at_offset($_) for $begin, $end;
-	if ($step eq 'I') { # insert
-		$self->insert_blocks($begin, @data);
+	for my $step (@steps) {
+		my ($act, $begin, $end, @data) = @$step;
+		$act = $_reverse_step{$act} if $reverse;
+		#warn "Do: $act, $begin, $end, @data\n";
+		$_ = $self->get_iter_at_offset($_) for $begin, $end;
+		if ($act eq 'I') { # insert
+			$self->place_cursor($begin);
+			$self->insert_blocks_at_cursor(@data);
+		}
+		elsif ($act eq 'D') { # delete
+			my $tree = $self->get_parse_tree($begin, $end);
+			splice @$tree, 3, 0, $tree;
+				# replace insert data with actual deleted data
+				# these should be the same
+			$self->delete($begin, $end);
+			$self->place_cursor($end);
+		}
+		elsif ($act eq 'A') { # apply tag
+			$self->apply_tag($data[0], $begin, $end);
+		}
+		elsif ($act eq 'R') { # remove tag
+			$self->remove_tag($data[0], $begin, $end);
+		}
+		else { die "BUG: unknown step type: $act" }
 	}
-	elsif ($step eq 'D') { # delete
-		$self->delete($begin, $end);
-	}
-	elsif ($step eq 'A') { # add tag
-	}
-	elsif ($step eq 'R') { # remove tag
-	}
-	else { die "BUG: unknown step type: $step" }
+	$self->set_modified(1);
 	$self->{_undo_lock} = 0;
 }
 
