@@ -4,17 +4,21 @@ use strict;
 use utf8; # for translator credits
 
 use Carp;
-use UNIVERSAL qw/isa/;
-use File::Spec;
-use File::BaseDir qw/
-	xdg_config_home xdg_config_files
-	xdg_data_home xdg_data_dirs xdg_data_files/;
+use File::BaseDir 0.03 qw/
+	config_home config_files
+	data_files data_dirs
+	xdg_cache_home
+/;
 use Zim::Utils;
+use Zim::Events;
 use Zim::Store;
 use Zim::History;
 use Zim::Page;
 
-our $VERSION = '0.23';
+our $VERSION = '0.25';
+
+our @ISA = qw/Zim::Events/;
+
 our $COPYRIGHT = 'Copyright (c) 2005, 2007 Jaap G Karssenberg.';
 our $LONG_VERSION = << "EOT";
 zim $VERSION - A desktop wiki and outliner
@@ -30,16 +34,32 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 Please report bugs to pardus\@cpan.org
 EOT
 our $AUTHORS = 'Jaap Karssenberg <pardus@cpan.org>';
-our $TRANSLATORS =
-'Jaap Karssenberg (nl)
-Vlastimil Ott (cs)
+our $TRANSLATORS = q{
+Andrés Rassol (es)
+Balaam's Miracle (nl)
+Daniel Stoyanov (fr)
+Etienne Le Belléguy (fr)
+Frederik 'Freso' S. Olesen (da)
+Frits Salomons (nl)
+Jaap Karssenberg (nl)
+Jacopo Moronato (it)
 Javier Rovegno Campos (es)
-Francisco Molinero (es)
-Mikael Mildén (sv)
+Jeppe Toustrup (da)
 João Santos (pt)
-Etienne LB (fr)
+Jérôme Guelfucci (fr)
+Klaus Vormweg (de)
+Matthias Mailänder (de)
+Mikael Mildén (sv)
 Nucleos (fr)
-Jacopo Moronato (it)';
+Paco Molinero (es)
+René 'Necoro' Neumann (de)
+Roberto Suarez (gl)
+Rui Nibau (fr)
+Vladimir Sharshov (ru)
+Vlastimil Ott (cs)
+Yaron (he)
+sanya (ru)
+};
 our $WEBSITE = 'http://www.pardus.nl/projects/zim';
 
 =head1 NAME
@@ -50,10 +70,7 @@ Zim - Application object for the zim desktop wiki
 
 	use Zim;
 	
-	my $repo = Zim->new(
-		type     => 'Files',
-		dir      => $dir,
-	);
+	my $repo = Zim->new( dir => $dir );
 	my $page = $repo->get_page('Home');
 
 =head1 DESCRIPTION
@@ -67,6 +84,19 @@ These repositories are called "notebooks" in the UI
 The interface which child objects should support is defined in
 L<Zim::Store>.
 
+=head1 SIGNALS
+
+This following signals are emitted by this object:
+
+=over 4
+
+=item C<config_changed>
+
+Any othe object changing our config should call C<signal_emit> for this
+signal in order to notify all other objects.
+
+=back
+
 =head1 METHODS
 
 =head2 Public Methods
@@ -75,22 +105,14 @@ The following methods can be used by the GUI.
 
 =over 4
 
-=item new(PARENT, NAMESPACE, FILE)
+=item new(%PARAM)
 
-=item new(PARENT, NAMESPACE, DIR, TYPE, ...)
+Constructor. Parameters can be:
 
-Constructor. PARENT can be a parent repository object or undef.
-NAMESPACE is the prefix for all pages managed by this repository.
-
-TODO: implement this: FILE should be the path to a repository config file.
-
-DIR is the root namespace of the repository and might contain
-a onfig file.
-
-TYPE is optional and is used to initialize a top level child
-repository when no other config is given. This is used for example
-to intialize a repository. All remaining arguments are passed on
-to this top level child repository.
+	dir - top leve ldirectory for the notebook
+	type - type of the (default) top level store object
+	parent - parent object in case we are nested
+	namespace - namespace in case we are nested
 
 =cut
 
@@ -101,38 +123,59 @@ sub new {
 	$param{namespace} =~ s/:?$/:/;
 	die "BUG: store initialized without directory"
 		unless $param{dir};
-	$param{dir} = Zim::File->abs_path($param{dir});
-	die "Not a directory: $param{dir}\n"
-		if -e $param{dir} and ! -d $param{dir};
+	$param{dir} = dir($param{dir}) unless ref($param{dir});
+	die "No such directory: $param{dir}\n" unless $param{dir}->exists;
 	
 	my $self = bless {%param}, $class;
 	$self->{config} ||= {};
-	
-	# Find config
-	my ($conf) = grep $_->exists(),
-	             map  Zim::File->new($param{dir}, $_),
-		     qw/_zim.config .zim.config/;
-	$conf ||= Zim::File->new($param{dir}, '.zim.config');
-	$self->{conf_file} = $conf;
 
 	# Read config
-	$conf->read_config( $self->{config} );
-	for my $ns (grep /^:/, keys %{$self->{config}}) {
+	$$self{config_file} = $class->get_notebook_config($param{dir});
+	$$self{config_file}->read($$self{config}, 'Notebook');
+	$self->signal_connect(config_changed => \&_init_properties, $self);
+	$self->_init_properties;
+
+	# Set default store if not yet defined
+	my $type = $self->{config}{type} || 'Files';
+	$self->add_child(':', $type, %param)
+		unless defined $self->{config}{':'};
+
+	# Initialize stores defined in config
+	for my $ns (sort keys %{$self->{config}{Namespaces}}) {
 		# remove namespace defnitions from config
 		# allows lines like ":namespace=Class,key=val,key=val"
-		my $val = delete $self->{config}{$ns};
+		my $val = $self->{config}{Namespaces}{$ns};
 		my ($class, $arg) = split ',', $val, 2;
 		my %arg = map split('=',$_,2),
 		          map split(',',$_,2), $arg; 
 		$self->add_child($ns => $class, %arg);
 	}
 	
-	# Set default store if not yet defined
-	my $type = $self->{type} || 'Files';
-	$self->add_child(':', $type, %param)
-		unless defined $self->{':'};
+	return $self;
+}
 
-	return $self;	
+sub _init_properties {
+	my $self = pop; # pop because we are called from signal
+	my ($dir, $config) = @$self{'dir', 'config'};
+
+	$$dir{slow_fs} = $$config{slow_fs};
+
+	if ($$config{DirType}) {
+		my $class = 'Zim::FS::'.$$config{DirType};
+		eval "use $class"; die $@ if $@;
+		$$self{dir} = $class->new($dir);
+		$dir = $$self{dir};
+	}
+	warn "## root uses dir: $$self{dir} (".ref($$self{dir}).")\n";
+
+	my $doc_root = $$config{document_root};
+	$doc_root = dir( Zim::FS->abs_path($doc_root, $dir) )
+		if defined $doc_root and ! ref $doc_root;
+	# TODO make sure that if dir falls below document_dir, or document_dir
+	#      below dir they are properly nested. Also respect if
+	#      either one is under a VCS 
+	#      ... need auto-detect VCS instead of "DirType" ?
+	$$self{document_root} = $doc_root;
 }
 
 =item C<init_history()>
@@ -144,23 +187,13 @@ Returns the history object or undef.
 
 sub init_history {
 	my $self = shift;
-
-	my $hfile = $self->{config}{hist_file};
-	unless ($hfile) { # default hist file
-		$hfile = Zim::File->abs_path(
-			$self->{dir}.'/.zim.history.cache');
-		$hfile = Zim::File->cache_path($hfile)
-			if -e $self->{dir} and ! -w $self->{dir};
-	}
+	my $dir = Zim->get_notebook_cache($$self{dir});
+	my $file = $dir->file('history.cache');
 	
-	my $hdir = Zim::File->dir($hfile);
-	$hfile = undef
-		if (-e $hfile and ! -w $hfile)
-		or (-e $hdir  and ! -w $hdir ) ;
-	warn "# History file: $hfile\n";
+	warn "# History file: $file\n";
 	my $max = $self->{config}{hist_max} || 15;
 	eval {
-		$self->{history} ||= Zim::History->new($hfile, $max, undef);
+		$self->{history} ||= Zim::History->new($file, $max, undef);
 	};
 	warn $@ if $@;
 	
@@ -205,14 +238,6 @@ sub add_child {
 	#warn "regex: $self->{regex}\n";
 }
 
-=item C<config()>
-
-Returns a hash with config options.
-
-=cut
-
-sub config { $_[0]->{config} }
-
 =item C<save()>
 
 Save config, history, etc. to disk.
@@ -220,14 +245,11 @@ Save config, history, etc. to disk.
 =cut
 
 sub save {
-	# Need to merge config with definition for stores
 	my $self = shift;
-	my $file = Zim::File->new($self->{conf_file});
-	my $conf = $file->read_config;
-	! /^:/ && delete $$conf{$_} for keys %$conf;
-	$$conf{$_} = $$self{config}{$_} for keys %{$$self{config}};
-	$file->write_config($conf);
+	return 0 if $$self{config}{read_only};
+	$$self{config_file}->write($$self{config}, 'Notebook');
 	$self->{history}->write if $self->{history};
+	return 1;
 }
 
 =item C<root()>
@@ -254,7 +276,7 @@ sub base { return $_[0]->{dir} }
 sub _check_page {
 	if (ref $_[0]) {
 		croak "Object \"$_[0]\" is not a Zim::Page"
-	       		unless isa $_[0], 'Zim::Page';
+	       		unless $_[0]->isa('Zim::Page');
 	}
 	elsif ($_[1]) { croak "\"$_[0]\" is not a page object" }
 	else { goto \&_check_page_name }
@@ -411,9 +433,11 @@ sub _clone { # Copy and move are almost the same
 	_check_page($source);
 	_check_page($target);
 	
-	$source = $self->get_page($source) || die "Could not open page '$source'\n"
+	$source = $self->get_page($source)
+		|| die "Could not open page '$source'\n"
 		unless ref $source;
-	$target = $self->get_page($target) || die "Could not create page '$target'\n"
+	$target = $self->get_page($target)
+		|| die "Could not create page '$target'\n"
 	       	unless ref $target;
 	
 	die "BUG: You tried to move a page marked as read-only\n"
@@ -428,7 +452,7 @@ sub _clone { # Copy and move are almost the same
 		$source->{store}->$method($source, $target, $update);
 	}
 	else {
-		$target->clone($source);
+		$target->clone($source, media => 'none');
 		if ($update) {
 			my ($from, $to) = ($source->name, $target->name);
 			$self->get_page($_)->update_links($from => $to)
@@ -515,27 +539,27 @@ sub interwiki_lookup {
 	else { # file lookup
 		# TODO repository specific urls here
 		
-		# Lookup named zim repositories
-		my $dir = $class->get_notebook($key);
-		if (defined $dir) {
-			$url = Zim::File->path2uri($dir);
-			$url =~ s#^file:#zim:#i;
-			$url .= '?{NAME}';
+		if ($class->get_notebook($key)) {
+			# Named zim notebook - put key in url, not actual dir !
+			$url = "zim://$key?{NAME}";
 		}
-
-		# lookup interwiki config
-		unless (length $url) {
-			for (xdg_data_files(qw/zim urls.list/)) {
-				$url = Zim::File->new($_)->read_hash($key);
-				last if defined $url;
+		else {
+			# lookup interwiki config
+			for (data_files(qw/zim urls.list/)) {
+				for (file($_)->read) {
+					next unless /^\Q$key\E\s+(\S+)/;
+					$url = $1;
+					last;
+				}
 			}
 		}
 	}
-	return unless length $url;
+	return unless defined $url and length $url;
 	$_urls{lc($key)} = $url;
 
 	unless ($url =~ s/{NAME}/$page/) {
-		$page =~ s/([^A-Za-z0-9\-_.!~*'()])/sprintf("%%%02X", ord($1))/ge;
+		$page =~ s/([^A-Za-z0-9\-_.!~*'()])/
+				sprintf("%%%02X", ord($1))/gex;
 		$url =~ s/{URL}/$page/ or $url .= $page;
 	}
 	
@@ -562,77 +586,205 @@ sub _flush_cache { # TODO TODO good API for this
 
 =back
 
+=head2 Document Interface
+
+This interface is used to deal with external documents that belong to the
+notebook. Think of these documents as attachments.
+
+See detailed docs in L<Zim::Store>.
+
+=over 4
+
+=item C<document_dir(PAGE)>
+
+Returns the document dir for PAGE.
+
+=cut
+
+sub document_dir {
+	my ($self, $page) = @_;
+	return $$self{dir} unless $page;
+	_check_page($page);
+	$page = $self->get_page($page) unless ref $page;
+	$$page{store}->document_dir($page);
+}
+
+=item C<document_root()>
+
+Returns the document root or undef.
+
+=cut
+
+sub document_root { $_[0]{document_root} }
+
+=item C<store_file(FILE, PAGE)>
+
+Stores FILE in the document dir.
+
+=cut
+
+sub store_file {
+	my ($self, $file, $page) = @_;
+	return Zim::Store::store_file($self, $file) unless defined $page;
+	_check_page($page);
+	$page = $self->get_page($page) unless ref $page;
+	$$page{store}->store_file($file, $page);
+}
+
+=item C<resolve_file(PATH, PAGE)>
+
+Resolve a linked file.
+
+=cut
+
+sub resolve_file {
+	my ($self, $file, $page) = @_;
+	return Zim::Store::resolve_file($self, $file) unless defined $page;
+	_check_page($page);
+	$page = $self->get_page($page) unless ref $page;
+	$$page{store}->resolve_file($file, $page);
+}
+
+=item C<relative_path(PATH, PAGE)>
+
+Returns a relative path for creating links.
+
+=cut
+
+sub relative_path {
+	my ($self, $file, $page) = @_;
+	return Zim::Store::relative_path($self, $file) unless defined $page;
+	_check_page($page);
+	$page = $self->get_page($page) unless ref $page;
+	$$page{store}->relative_path($file, $page);
+}
+
+=back
+
 =head2 Class Methods
 
 These methods can be called as C<< Zim->method(...) >>.
 
 =over 4
 
-=item C<list_notebooks()>
+=item C<get_notebook_config(DIR)>
 
-Returns a reference to a list with names for known notebooks.
+Return a config file for the notebook at DIR.
+
+=item C<get_notebook_cache(DIR)>
+
+Returns a dir to store e.g. history and other state files.
+By default this is either ".zim/" in the notebook directory
+or a directory under XDG_CACHE.
 
 =cut
 
-sub list_notebooks {
-	my $class = shift;
-	my (undef, $list) = _read_rep_list() or return ();
-	@$list = grep {! /^_/} @$list;
-	return $list;
+sub get_notebook_config {
+	my ($class, $dir) = @_;
+	my ($file) = grep {-e $_}
+	             map  "$dir/$_", qw/notebook.zim .notebook.zim/;
+	$file = "$dir/notebook.zim" unless defined $file;
+	return Zim::FS::File::Config->new($file);
 }
 
-=item C<set_default_notebook(REPOSITORY)>
+sub get_notebook_cache {
+	my ($class, $dir) = @_;
 
-Set the path or name for the default notebook.
+	# If dir is an object already it can have a property "slow_fs"
+	# FIXME should we check notebook.zim for this property if we
+	# get a path ?
+	unless (ref($dir) and $$dir{slow_fs}) {
+		my $cache = dir( $dir."/.zim" );
+		# do not use subdir here - cache should e.g. not be under VCS
+		return $cache if $cache->writable;
+	}
+	
+	# logic similar to login in cache_file() in FS.pm
+	my $name = Zim::FS->abs_path($dir);
+	$name =~ s/[\/\\:]+/_/g; # win32 save
+	$name =~ s/^_+|_+$//g;
+	return dir(xdg_cache_home(), 'zim', $name);
+}
 
-=item C<get_default_notebook()>
+=item C<is_notebook(DIR)>
 
-Returns the path for the default notebook, if there is one.
+Returns true if DIR is the root of a notebook.
 
 =cut
 
-sub set_default_notebook { $_[0]->set_notebook(_default_ => $_[1]) }
-
-sub get_default_notebook {
-	my $class = shift;
-	my $def = $class->get_notebook('_default_') or return undef;
-	return $def if $def =~ /[\/\\]/;
-	return $class->get_notebook($def);
+sub is_notebook {
+	my ($class, $dir) = @_;
+	return $class->get_notebook_config($dir)->exists;
 }
 
-=item C<set_notebook(NAME => PATH)>
-
-Set the name for a notebook.
-
 =item C<get_notebook(NAME)>
 
-Returns a path for a notebook with NAME or undef if the notebook is unknown.
-The lookup is case in-sensitive.
+Returns a path for notebook NAME. Special names are:
+
+	_default_	default notebook
+	_doc_		help manual
+
+=item C<get_notebook_list()>
+
+Returns a list of array references containing name and path for known notebooks.
+
+=item C<set_notebook_list()>
+
+Save a list of array references containing name and path for known notebooks.
 
 =cut
 
-sub set_notebook {
-	my ($class, $name, $path) = @_;
-	my ($hash, $list) = _read_rep_list();
-	$hash ||= {};
-	$list ||= [];
-	die "TODO";
-}
-
 sub get_notebook {
 	my ($class, $name) = @_;
-	my ($hash) = _read_rep_list() or return undef;
-	($name) = grep {lc($_) eq lc($name)} keys %$hash
-		unless exists $$hash{$name};
-	return $name ? $$hash{$name} : undef;
+	
+	if ($name eq '_doc_') {
+		my $path = data_dirs(qw/zim doc/);
+		return dir( $path );
+	}
+
+	my ($n, $m);
+	my @list = reverse $class->get_notebook_list;
+	for (@list) {
+		# do lookup both exact and case-insensitive
+		if    ($$_[0] eq $name        ) { $n = $$_[1] }
+		elsif (lc($$_[0]) eq lc($name)) { $m = $$_[1] }
+	}
+	$n ||= $m;
+	return undef unless $n;
+
+	return $class->get_notebook($n)
+		if $name eq '_default_' and $n !~ /[\\\/]/;
+		# recurs when default is not a path but a name
+
+	return dir($n) ;
 }
 
-sub _read_rep_list {
-	my ($file) = xdg_config_files('zim', 'repositories.list');
+sub get_notebook_list {
+	shift; # class
+	my $file = config_files('zim', 'notebooks.list');
+	$file ||= config_files('zim', 'repositories.list');
+		# backwards compatibility to versions < 0.23
 	return () unless defined $file;
-	return Zim::File->new($file)->read_config;
+	my @notebooks = grep defined($_), map {
+		/^((?:\\.|\S)+?)[=\s]+(.+?)\r?\n/
+			? [$1 => $2] : undef ;
+	} file($file)->read;
+		# allow "=" as separator instead of whitespace
+		# for backwards compatibility to versions < 0.24
+	$$_[0] =~ s/\\([\s\\])/$1/g for @notebooks; # unescape whitespace
+	return @notebooks
 }
 
+sub set_notebook_list {
+	shift; # class
+	my @lines = map {
+		my ($name, $path) = @$_;
+		$name =~ s/([\s\\])/\\$1/g; # escape whitespace
+		"$name\t$path\n";
+	} @_;
+	my $file = config_home('zim', 'notebooks.list');
+	file($file)->write(@lines);
+}
 
 1;
 
