@@ -2,7 +2,7 @@ package Zim::Events;
 
 use strict;
 
-our $VERSION = '0.18';
+our $VERSION = '0.24';
 
 =head1 NAME
 
@@ -32,7 +32,7 @@ sub signal_connect {
 
 sub signal_emit {
 	my ($self, $signal, @args) = @_;
-	warn "## ".ref($self)."::signal_emit $signal @args\n";
+	warn "## emit $signal: @args (".ref($self).")\n";
 	return unless exists $self->{_signals}{$signal};
 	for my $handler (grep defined($_), @{$self->{_signals}{$signal}}) {
 		my ($code, @data) = @$handler;
