@@ -5,7 +5,7 @@ use Encode;
 use Zim::Template;
 use Zim::Formats;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 our @ISA = qw/Zim::Formats/;
 
 =head1 NAME
@@ -36,11 +36,8 @@ sub save_tree { # HACK: template and temp_data are unsupported args
 	my ($class, $io, $tree, $page, $template, $temp_data) = @_;
 	#use Data::Dumper; warn Dumper $tree;
 
-	$template ||= $page->{store}{_template} || '';
-	$template = length($template)
-		? Zim::Template->new($template)
-		: Zim::Template->new(\'[% page.body %]')
-		unless ref $template ;
+	$template ||= $page->{store}{_template}
+	          ||  Zim::Template->new(\'[% page.body %]');
 
 	my ($title, $heading) = ($page->basename, '');
 	if (ref $$tree[2] and $$tree[2][0] =~ /^head/) {
@@ -58,8 +55,8 @@ sub save_tree { # HACK: template and temp_data are unsupported args
 
 	my $data = {
 		page => {
-			name => '',
-			namespace => '',
+			name => $page->name,
+			namespace => $page->namespace,
 			title => $title,
 			heading => $heading,
 			is_index => ($page->{_is_index} ? 1 : 0),
@@ -120,8 +117,10 @@ sub _dump_node {
 		@tags = ("<p>\n", "\n</p>\n");
 		if (grep {!ref($_) and /^\s*\x{2022}\s+/m} @$node) {
 			# ugly hack ... should not be necessary
+			print $tags[0];
 			$class->parse_list($page, @$node);
-			@$node = ();
+			print $tags[1];
+			return;
 		}
 		elsif ( $page->{_mode} eq 's5'
 				and ! ref $$node[0]
@@ -140,20 +139,19 @@ sub _dump_node {
 	elsif ($name eq 'link') {
 		my ($type, $href) = $page->parse_link($$opt{to});
 		if ($type eq 'page') {
-			my ($l, $n) = href($page,
-				$page->resolve_name($href), $$opt{to});
+			my ($l, $n) = href($page, $href, $$opt{to});
 			$href = $l;
 			$$node[0] = $n
 				if @$node == 1 and $$node[0] eq $$opt{to};
 		}
 		elsif ($type eq 'file') {
-			$href = href_file($href);
+			$href = href_file($page, $$opt{to}, $href);
 		}
 		warn "#  link: $$opt{to} => $href\n";
 		@tags = ("<a href='$href'>", "</a>");
 	}
 	elsif ($name eq 'image') {
-		my $src = href_file($$opt{src});
+		my $src = href_file($page, @$opt{'src', 'file'});
 		my $alt = $src;
 		$alt =~ s#.*[/\\]##g;
 		@tags = ("<img src='$src' alt='$name' />", '');
@@ -176,14 +174,27 @@ sub _dump_node {
 
 Function that returns both an url and a name for a link.
 
-=item C<href_file(FILE)>
+=item C<href_file(PAGE, HREF, FILE)>
 
-Function that returns an url for a file link.
+Function that returns a path or url for a file link.
+HREF is the (relative) link, FILE is an optional absolute path.
 
 =cut
 
 sub href_file {
-	my $href = shift;
+	my ($page, $href, $file) = @_;
+
+	if ($href =~ /^\./) { # relative paths most be relative to this file
+		$file ||= $page->resolve_file($href);
+		$href = Zim::FS->rel_path($file, $$page{source}->dir, 'UP');
+	}
+	elsif ($href =~ /^\//) { # maybe we use a document root
+		$href = $file if defined $file;
+	}
+	elsif ($href =~ /^\~/) { # html doesn't know about home dir
+		$href = $file || Zim::FS->abs_path($href);
+	}
+
 	$href = Encode::encode_utf8($href); # turn utf8 flag off
 	$href =~ s{ ([^A-Za-z0-9\-\_\.\!\~\*\'\(\)\/\:]) }
 	          { sprintf("%%%02X",ord($1))            }egx;
@@ -217,7 +228,10 @@ sub href { # only links to other pages
 	return ($href, $name);
 }
 
-sub parse_list { # FIXME wrong level for this kind of parsing :(
+sub parse_list {
+	# FIXME wrong level for this kind of parsing :(
+	# FIXME this logic assumes 1 whitespace is 1 level of indenting
+	# this does not work when indenting with space instead of tab
 	my $class = shift;
 	no warnings;
 
