@@ -6,7 +6,7 @@ use POSIX qw(strftime);
 use Zim::Template;
 use Zim::Formats;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 our @ISA = qw/Zim::Formats/;
 
 =head1 NAME
@@ -120,7 +120,7 @@ sub _dump_node {
 	elsif ($name eq 'link') {
 		my ($type, $href) = $page->parse_link($$opt{to});
 		if ($type eq 'page') {
-			$href = _href($page, $page->resolve_name($href));
+			$href = _href($page, $href);
 		}
 		print STDERR "  link: $$opt{to} => $href\n";
 		@tags = ('[', " $href]");
