@@ -2,6 +2,7 @@ package Zim::GUI::PageView;
 
 use strict;
 use vars qw/$CODESET/;
+use Carp;
 use POSIX qw(strftime);
 use Encode;
 use Gtk2;
@@ -9,17 +10,16 @@ use Gtk2::Pango;               # pango constants
 use Gtk2::Ex::HyperTextView;   # custom widget
 use Gtk2::Ex::HyperTextBuffer; #    "     "
 use Gtk2::Gdk::Keysyms;	       # key identifiers
-use File::Spec;
 use File::MimeInfo q/mimetype_isa/;
 use File::MimeInfo::Magic qw/mimetype/;
-use File::BaseDir qw/xdg_data_files xdg_data_home/;
+use File::BaseDir 0.03 qw/data_files data_home/;
 use Zim::Utils;
 use Zim::GUI::Component;
 
 eval "use Gtk2::Ex::DesktopEntryMenu";
 my $has_mimeapplications = $@ ? 0 : 1;
 
-our $VERSION = '0.23';
+our $VERSION = '0.25';
 
 our @ISA = qw/Zim::GUI::Component/;
 
@@ -81,8 +81,9 @@ Delete		gtk-delete	_Delete		.		Delete
 EditObject	gtk-properties	_Edit Link...	<ctrl>E		Edit link
 InsertDate	.		_Date and Time...	<ctrl>D	Insert date
 InsertLink	gtk-connect	_Link...		.	Insert link
+InsertExtLink	gtk-connect	E_xternal Link...	.	Insert external link
 InsertImage	.		_Image...	.	Insert image
-InsertFromFile	.		_From File...	.	Insert from file
+InsertFromFile	.		Text _From File...	.	Insert text from file
 FindReplace	gtk-find-and-replace	_Replace...	<ctrl>H	Find and Replace
 } );
 
@@ -143,6 +144,7 @@ my $ui_layout = q{<ui>
 			<placeholder name='PluginItems'/>
 			<separator/>
 			<menuitem action='InsertFromFile'/>
+			<menuitem action='InsertExtLink'/>
 			<menuitem action='InsertLink'/>
 		</menu>
 		<menu action='FormatMenu'>
@@ -175,7 +177,6 @@ my $ui_layout = q{<ui>
 			<toolitem action='TUnderline'/>
 			<toolitem action='TStrike'/>
 		</placeholder>
-		<separator/>
 	</toolbar>
 </ui>};
 
@@ -215,25 +216,22 @@ sub init { # called by new()
 	my $self = shift;
 
 	# load style data
-	my $file = Zim::GUI::_xdg_config_data_files('zim', 'style.conf');
-	my $tags = Zim::File->new($file)->_read_config_alt;
-	if (%$tags) {
+	my $profile = $$self{app}{profile} || 'default';
+	my $file = Zim::GUI::_config_data_files('zim', $profile.'.style');
+	$file ||= Zim::GUI::_config_data_files('zim', 'default.style');
+	my $style = Zim::FS::File::Config->new($file)->read(undef, 'TextView')
+		if $file;
+	for my $k (grep s/^Tag //, keys %$style) {
 		no strict 'refs';
-		for my $k (keys %$tags) {
-			my $list = [
-				map { # map constants
-					/^PANGO_/ ? &{"Gtk2::Pango::$_"} : $_
-				}
-				%{$$tags{$k}}
-			];
-			$Gtk2::Ex::HyperTextBuffer::TAGS{$k} = $list;
-		}
+		my $list = [
+			map { # map constants
+				/^PANGO_/ ? &{"Gtk2::Pango::$_"} : $_
+			}
+			%{$$style{"Tag $k"}}
+		];
+		$Gtk2::Ex::HyperTextBuffer::TAGS{$k} = $list;
 	}
-	else {
-		@Gtk2::Ex::HyperTextBuffer::TAGS{'link', 'underline'}
-			= ([foreground => 'blue'], [background => 'yellow']) ;
-	}
-
+	#use Data::Dumper; warn Dumper \%Gtk2::Ex::HyperTextBuffer::TAGS;
 
 	my $vbox = Gtk2::VBox->new(0, 0);
 	$self->{vbox} = $vbox;
@@ -251,14 +249,15 @@ sub init { # called by new()
 	$htext->set_right_margin(5);
 	$htext->set_tabs( Gtk2::Pango::TabArray->new_with_positions(
 		# initial_size, pos_in_pixels, ... allign => position
-		1, 0, 'left' => 40 * PANGO_SCALE ) );
+		1, 1, 'left' => $$style{tabs} ) )
+		if $$style{tabs};
 	$htext->{follow_on_enter} = $self->{app}{settings}{follow_on_enter};
 	$htext->signal_connect(link_clicked =>
 		sub { $self->{app}->link_clicked($_[1]) }  );
 	$htext->signal_connect(link_enter =>
 		sub {
 			my $p = $self->{app}{page};
-			my ($t, $l) = $p->parse_link($_[1]);
+			my ($t, $l) = $p->parse_link($_[1], 'NO_RESOLVE');
 			#warn "parsed: $_[1] => $l ($t)\n";
 			$l = $p->resolve_name($l) if $t eq 'page';
 			$self->{app}->push_status("Go to \"$l\"", 'link');
@@ -308,7 +307,7 @@ sub init { # called by new()
 		clicked => \&CloseFind, $self );
 	$hbox->pack_end($close_button, 0,1,0);
 	
-	$hbox->pack_start( Gtk2::Label->new(' Find: '), 0,1,0);
+	$hbox->pack_start( Gtk2::Label->new(' '.__('Find').': '), 0,1,0); #. query entry label
 	
 	my $entry = Gtk2::Entry->new();
 	$entry->signal_connect(changed => sub { $self->_find($entry->get_text, 0) });
@@ -316,13 +315,13 @@ sub init { # called by new()
 	$hbox->pack_start($entry, 0, 1, 0);
 	$self->{entry} = $entry;
 
-	my $prev_button = $self->new_button('gtk-go-back', '_Previous');
+	my $prev_button = $self->new_button('gtk-go-back', __('_Previous')); #. search button
 	$prev_button->set_sensitive(0);
 	$prev_button->signal_connect_swapped(clicked => \&FindPrev, $self);
 	$hbox->pack_start($prev_button, 0, 1, 0);
 	$self->{find_prev_button} = $prev_button;
 	
-	my $next_button = $self->new_button('gtk-go-forward', '_Next');
+	my $next_button = $self->new_button('gtk-go-forward', __('_Next')); #. search button
 	$next_button->set_sensitive(0);
 	$next_button->signal_connect_swapped(clicked => \&FindNext, $self);
 	$hbox->pack_start($next_button, 0, 1, 0);
@@ -335,6 +334,7 @@ sub init { # called by new()
 	
 	my $accels = $self->{app}{ui}->get_accel_group;
 	unless ($self->{app}{read_only}) {
+		# we do not show these actions in global read_only mode
 		# new action group will be initialized
 		$self->add_actions($ui_format_actions, undef, 'ApplyFormat');
 		$self->add_actions($ui_format_toggle_actions, 'TOGGLE', 'ApplyFormat');
@@ -349,21 +349,46 @@ sub init { # called by new()
 	$accels->connect( $k_F3, [], ['visible'], sub { $self->on_FindNext } );
 	$accels->connect( $k_F3, ['shift-mask'], ['visible'], sub { $self->on_FindPrev } );
 
-	# FIXME - prevent our accelerators from triggering while editing another widget
+	# FIXME - prevent our accelerators from triggering while 
+	# another widget has focus 
 	if (my $actions = $self->{actions}) {
 		$htext->signal_connect(
-			focus_in_event  => sub { $actions->set_sensitive(1) unless $self->{read_only} });
+			focus_in_event  => sub { $actions->set_sensitive(1)
+				unless $$self{app}{read_only} });
 		$htext->signal_connect(
 			focus_out_event => sub { $actions->set_sensitive(0) });
 	}
+
+	$$self{app}->signal_connect(t_read_only => sub {
+		my ($app, $ro) = @_;
+		$$self{actions}->set_sensitive(!$ro) if defined $$self{actions};
+		$$self{htext}->set_editable(!$ro);
+		$self->on_show_cursor;
+	} );
 	
-	my $sendto_dir = File::Spec->catdir(xdg_data_home(), 'zim', 'SendTo');
+	my $sendto_dir = data_home('zim', 'SendTo');
 	$self->{sendto_dir} = $sendto_dir if -d $sendto_dir;
 	#warn "sendTo dir: $sendto_dir\n";
 	
 	$self->on_show_cursor;
 }
 
+sub _init_buffer {
+	# create a new HyperTextBuffer
+	# called by load_page
+	my $self = shift;
+	my $buffer = Gtk2::Ex::HyperTextBuffer->new();
+	$buffer->create_default_tags;
+	$self->{buffer} = $buffer;
+	$self->{htext}->set_buffer($buffer);
+	$buffer->signal_connect(modified_changed =>
+		sub {$self->{app}->update_status} );
+	$buffer->signal_connect(edit_mode_changed =>
+		\&on_edit_mode_changed, $self );
+	$self->on_show_cursor;
+	$buffer->{_zim_name} = $self->{app}{name}; # FIXME FIXME FIXME
+}
+
 =back
 
 =head2 Actions
@@ -511,7 +536,8 @@ sub Link {
 	$link = $text unless defined $link;
 	return if $link =~ /\n/;
 	
-	unless ($self->{read_only}) {
+	my $ro = $$self{app}{read_only};
+	unless ($ro) {
 		return 0 if defined $bool and !$bool;
 		my $bit = $link eq $text;
 		$buffer->remove_all_tags($start, $end);
@@ -519,10 +545,12 @@ sub Link {
 		$buffer->set_modified(1);
 	}
 
-	my ($type, $l) = $self->{app}{page}->parse_link($link);
-	$self->{app}->link_clicked($link)
-		if $self->{read_only}
-		or $type eq 'page' and $self->{app}{settings}{follow_new_link};
+	if ($ro or $$self{app}{settings}{follow_new_link}) {
+		my ($type, $l) = $$self{app}{page}->parse_link(
+			$link, 'NO_RESOLVE');
+		$$self{app}->link_clicked($link)
+			if $type eq 'page';
+	}
 }
 
 =item C<Undo()>
@@ -559,10 +587,14 @@ Delete from cursor.
 
 =cut
 
+sub on_Delete {
+	my ($action, $self) = @_;
+	$self->Delete if $$self{htext}->get('has-focus');
+}
+
 sub Delete {
 	my $self = shift;
-	return unless $self->{htext}->get('has-focus');
-	$self->{htext}->signal_emit('delete_from_cursor', 'chars', 1);
+	$$self{htext}->signal_emit('delete_from_cursor', 'chars', 1);
 }
 
 =item C<EditObject(ITER)>
@@ -618,7 +650,9 @@ sub EditLink {
 	my $text = $buffer->get_text($start, $end, 0);
 	$text = undef if $text =~ /\n/;
 
-	($link, $text) = $self->_prompt_link_properties($link, $text, 'edit');
+	my ($t, undef) = $$self{app}{page}->parse_link($link, 'NO_RESOLVE');
+	($link, $text) = $self->_prompt_link_properties(
+		$link, $text, 'edit', ($t eq 'page') ? 0 : 1);
 	return unless defined $link;
 	
 	$buffer->delete($start, $end);
@@ -678,12 +712,12 @@ sub Find {
 	unless ($self->{entry}->visible) {
 		$self->{hbox}->set_no_show_all(0);
 		$self->{hbox}->show_all;
-		$self->{entry}->grab_focus;
 	}
 	if (length $query) {
 		$self->{entry}->set_text($query);
 		$self->_find($query, 0);
 	}
+	$self->{entry}->grab_focus;
 }
 
 sub FindNext { $_[0]->_find( $_[0]->{entry}->get_text, 1  ) }
@@ -737,8 +771,10 @@ sub InsertDate { # insert date that is linked
 
 	my $buffer = $self->get_buffer;
 	my $string = Encode::decode($CODESET, strftime($fmt, @$date));
-	my $link = $self->{app}{settings}{cal_namespace}
-		. strftime('%Y_%m_%d', @$date) ;
+	my $ns = $self->{app}{settings}{'Calendar Plugin'}{Namespace}
+		|| ':Date:' ;
+		# FIXME HACK - should not hardcode plugin here !
+	my $link = $ns . strftime('%Y:%m:%d', @$date) ;
 	
 	$buffer->insert_blocks_at_cursor(
 		($l ? ['link', {to => $link}, $string] : $string), ' ' );
@@ -768,16 +804,16 @@ sub prompt_date_format {
 	$check->set_active($l);
 	$dialog->vbox->add($check);
 
-	my ($file) = xdg_data_files('zim', 'dates.list');
+	my ($file) = data_files('zim', 'dates.list');
 	my @dates;
 	if (defined $file) {
-		for (Zim::File->new($file)->read()) {
+		for (file($file)->read()) {
 			chomp;
 			! /^\s*#/ and /\S/ or next;
 			push @dates, $_;
 		}
 	}
-	else { @dates = ('%A %d/%m/%Y', '%A %d/%m/%Y %H:%M', '%c') }
+	else { @dates = ('%A %d/%m/%Y', '%A %d/%m/%Y %H:%M', '%c', '%x') }
 	@{$view->{data}} =
 		map Encode::decode($CODESET, strftime($_, @$date)),
 		@dates ;
@@ -804,41 +840,84 @@ sub prompt_date_format {
 =item C<InsertLink(LINK, TEXT, NO_ASK)>
 
 Insert LINK or prompt the user for a link.
+LINK can be either a page, a file or a url, however the dialog only has
+page name completion. See C<InsertExtLink> for an alternative interface.
 
 NO_ASK is a boolean to suppress the prompt.
 
+=item C<InsertExtLink(LINK, TEXT, NO_ASK)>
+
+Like C<InsertLink()> but shows a dialog with a browse button
+instead of page name completion.
+
 =cut
 
-sub InsertLink { 
-	my ($self, $link, $text, $no_ask) = @_;
+sub InsertLink {
+	# $ext is a private argument used for InsertExtLink
+	my ($self, $link, $text, $no_ask, $ext) = @_;
 	$text = $link unless length $text;
-	($link, $text) = $self->_prompt_link_properties($link, $text)
+	($link, $text) = $self->_prompt_link_properties($link, $text, 0, $ext)
 		unless defined $link and $no_ask;
+	my ($t, undef) = $$self{app}{page}->parse_link($link, 'NO_RESOLVE');
+	if ($t eq 'file') {
+		my $file = $self->check_file_input($link);
+		$file = $$self{app}{page}->relative_path($file);
+		$text = $file if $text eq $link;
+		$link = $file;
+	}
 	return unless defined $link;
 	$self->{buffer}->insert_blocks_at_cursor(
 		['link', {to => $link}, $text], ' ' );
 }
 
-=item C<InsertImage(IMAGE, NO_ASK)>
+sub InsertExtLink {
+	my ($self, $link, $text, $no_ask) = @_;
+	return $self->InsertLink($link, $text, $no_ask, 'EXTERNAL');
+}
 
-Inserts an image into the buffer.
-Without argument prompts the user for a file.
+=item C<InsertImage(PATH, NO_ASK)>
 
-Unless NO_ASK is set it gives a dialog to confirm the action
-which allows to set any properties for the image.
+Inserts an image into the buffer. PATH is an optional path for the
+image. Unless NO_ASK is a dialog is presented which allows the user to set the
+path and other properties for the image. An error is raised when NO_ASK is set
+but no PATH is given.
+
+If PATH is given and it is not absolute we assume it to be a relative path
+returned by e.g. C<$page->store_file()>.
 
 =cut
 
 sub InsertImage {
-	my ($self, $file, $no_ask) = @_;
-	my %meta;
-	%meta = split /[=&]/, $1
-		if $file =~ s/\?(\w+=\w+(?:&\w+=\w+)*)$//;
-	(%meta) = (defined $file and $no_ask)
-		? (src => $file, %meta)
-		: $self->_prompt_image_properties(undef, src => $file) ;
-	return unless length $meta{src};
+	my ($self, $path, $no_ask) = @_;
+	croak "BUG: no image path given" if $no_ask and ! defined $path;
 
+	my %meta;
+	if (defined $path) {
+		# Image properties can be give in URL style
+		%meta = split /[=&]/, $1
+			if $path =~ s/\?(\w+=\w+(?:&\w+=\w+)*)$//;
+	
+		# The absolute path, $meta{file} is used to create the
+		# pixbuf etc. The human readable $meta{src} is used to
+		# present to the user and is the path that will be saved
+		# in the wiki source.
+		my $page = $$self{app}{page};
+		@meta{'file', 'src'} = Zim::FS->is_abs_path($path)
+				? ( $path, $page->relative_path($path) )
+				: ( $page->resolve_file($path), $path  ) ;
+	}
+	else {
+		$path = $self->filechooser_dialog(undef, 0, __('Insert Image')); #. dialog title
+		return unless defined $path;
+		$meta{src} = $path;
+	}
+
+	unless ($no_ask) {
+		%meta = $self->_prompt_image_properties(undef, %meta);
+		return unless length $meta{src};
+	}
+
+	# Insert image into buffer
 	$self->{buffer}->insert_blocks_at_cursor(['image', \%meta]);
 }
 
@@ -850,24 +929,20 @@ Insert contents of FILE as verbatim.
 
 sub InsertFromFile {
 	my ($self, $file, $no_ask) = @_;
-	unless ($no_ask) {
-		my $f = $self->run_prompt(
-			__('Insert from file'), #. dialog title
-			['file'], {file => [__('File'), 'file', '']}, #. label
-			undef, __('_Insert'), #. dialog button
-		);
-		return unless $f;
-		$file = $$f[0];
+	unless (defined $file and $no_ask) {
+		$file = $self->filechooser_dialog(
+			$file, 0, __('Insert from file') ); #. dialog title
+		return unless defined $file;
 	}
 	return $self->error_dialog(
 		__("No such file: {file}", file => $file))
 		unless -e $file;
-	my $mt = mimetype($file);
+	my $mt = mimetype("$file");
 	return $self->error_dialog(
-		__("File is not a text file: {file}", file =>$file))
+		__("File is not a text file: {file}", file =>$file)) #. error
 		unless mimetype_isa($mt, 'text/plain');
 	$$self{buffer}->insert_blocks_at_cursor(
-		['Verbatim', {}, ''.Zim::File->new($file)->read] );
+		['Verbatim', {}, ''.file($file)->read] );
 }
 
 sub on_key_press_event_hbox {
@@ -954,8 +1029,8 @@ sub WordCount {
 sub on_show_cursor {
 	my $self = shift;
 	$self->{htext}->set_cursor_visible(
-		! $self->{read_only}
-			|| $self->{app}{settings}{ro_cursor} );
+		! $$self{app}{read_only}
+			|| $$self{app}{settings}{ro_cursor} );
 }
 
 sub on_edit_mode_changed {
@@ -970,7 +1045,7 @@ sub on_edit_mode_changed {
 		my $data = $self->{htext}->get_link_at_cursor;
 		if (defined $data) {
 			my $p = $self->{app}{page};
-			my ($t, $l) = $p->parse_link($data);
+			my ($t, $l) = $p->parse_link($data, 'NO_RESOLVE');
 			$l = $p->resolve_name($l) if $t eq 'page';
 			$self->{app}->push_status(
 				__("Go to \"{link}\"", link => $l), 'link'); #. status bar info
@@ -984,7 +1059,7 @@ sub on_edit_mode_changed {
 		TItalic    => $tags{italic},
 		TUnderline => $tags{underline},
 		TStrike    => $tags{strike},
-	);
+	) if $$self{actions};
 	
 	my ($tag) = grep {$_ ne 'link'} keys(%tags), 'normal';
 	$$self{app}{statuss}->set_text($Styles{ucfirst($tag)});
@@ -1028,7 +1103,7 @@ sub on_drag_data_received {
 	else { # 'text/uri-list'
 		@blocks = map {
 			$_ = $self->{app}{page}->relative_path(
-					Zim::File->parse_uri($_) ) 
+					Zim::FS->path2uri($_) ) 
 				if m#^file:/#;
 			if (m#^\w[\w\+\-\.]+:/#) { # uri
 				['link', {to => $_}, $_], "\n";
@@ -1042,6 +1117,9 @@ sub on_drag_data_received {
 			}
 		} @items;
 	}
+
+	$$_[1]{file} = $$self{app}{page}->resolve_file($$_[1]{src})
+		for grep {ref $_ and $$_[0] eq 'image'} @blocks;
 	
 	($x, $y) = $self->{htext}->window_to_buffer_coords('widget', $x, $y);
 	my $iter = $self->{htext}->get_iter_at_location($x, $y);
@@ -1049,31 +1127,36 @@ sub on_drag_data_received {
 }
 
 sub on_populate_popup {
-	# add items to context menu
-	# object can be a link to a page, a URL (mail / file / http) or an image (which is a file)
-	# images can have a custom type that is handled by a plugin
+	# Add items to context menu. Object can be a link to a page, an URL 
+	# (mail / file / http) or an image (which is a file)
+	# images can have a custom type that is handled by a plugin.
+	# (For example the EquationEditor uses special images.)
 	my ($htext, $menu, $self) = @_;
 	my $iter = $htext->get_iter_at_pointer;
+	#warn "ITER ", $iter->get_offset, "\n";
 	return unless $iter;
 	
-	# source is the filename
-	# link is the name as written in the page source
-	my ($type, $source, $meta);
+	my ($type, $meta);
 	my $link = $htext->get_link_at_iter($iter);
 	if (defined $link) {
-		($type, $source) = $self->{app}{page}->parse_link($link);
+		($type, $link) = $self->{app}{page}->parse_link($link);
 	}
-	elsif (my $pixbuf = $iter->get_pixbuf) {
-		return unless defined $pixbuf->{image_src};
-		($type, $source) = ('image', $pixbuf->{image_file});
-		$meta = $pixbuf->{image_data};
+	else {
+		# Try to find a pixbuf, since clicking on an image can result
+		# in iters on both sides of the image we need to check also
+		# on char backward for an pixbuf.
+		my $pixbuf = $iter->get_pixbuf;
+		unless ($pixbuf) {
+			$iter->backward_char;
+			$pixbuf = $iter->get_pixbuf;
+		}
+		return unless $pixbuf and $$pixbuf{image_data};
+		$meta = $$pixbuf{image_data};
+		($type, $link) = ('image', $$meta{file});
 	}
-	else { return }
+	#warn "LINK: $link\n";
 	
-	# Separator
-	my $seperator = Gtk2::MenuItem->new();
-	$seperator->show;
-	$menu->prepend($seperator);
+	_separator($menu);
 	
 	# Edit object
 	my $edit_item = Gtk2::MenuItem->new(__('_Edit Link')); #. context menu
@@ -1081,6 +1164,8 @@ sub on_populate_popup {
 	$edit_item->signal_connect(activate => ($type eq 'image')
 		? sub { $self->EditImage($iter) }
 		: sub { $self->EditLink($iter)  }  );
+		# We can not use $link here - need link as typed,
+		# not as resolved. So we pass iter.
 	$menu->prepend($edit_item);
 
 	# Copy object
@@ -1092,6 +1177,7 @@ sub on_populate_popup {
 		# copy to both clipboard and selection
 		my $l = $link;
 		$l =~ s/^mailto://;
+		warn "## Copy to clipoard: $l\n";
 		for (qw/CLIPBOARD PRIMARY/) {
 			my $c = Gtk2::Clipboard->get(
 				Gtk2::Gdk::Atom->new($_) );
@@ -1101,41 +1187,39 @@ sub on_populate_popup {
 	$copy_item->show;
 	$menu->prepend($copy_item);
 
+	# Open Directory
+	if ($type eq 'image' or $type eq 'file') {
+		# This includes images with special type
+		my $browse_item = Gtk2::ImageMenuItem->new_with_mnemonic(
+			__('Open _Directory') ); #. context menu
+		$browse_item->set_image(
+			Gtk2::Image->new_from_stock('gtk-open', 'menu') );
+		$browse_item->signal_connect(activate =>
+			sub { $self->{app}->open_directory($link) } );
+		$browse_item->show;
+		$menu->prepend($browse_item);
+	}
+
 	if ($meta->{type} and exists $OBJECTS{$meta->{type}}) {
+		# Here we handle images that have special actions
+		# like equations etc.
 		my $obj = $OBJECTS{$meta->{type}};
 		$self->{app}->$obj->populate_popup($menu, $iter->get_pixbuf);
 	}
 	elsif ($type eq 'image' or $type eq 'file') {
-		# Open directory
-		my $browse_item = Gtk2::MenuItem->new(
-			__('Open _Directory') ); #. context menu
-		$browse_item->signal_connect(activate =>
-			sub { $self->{app}->open_directory($source) } );
-		$browse_item->show;
-		$menu->prepend($browse_item);
-		
-		return unless $has_mimeapplications
-			or $self->{sendto_dir} ;
-		
-		# Separator
-		$seperator = Gtk2::MenuItem->new();
-		$seperator->show;
-		$menu->prepend($seperator);
-	
+		_separator($menu);
+
 		# SendTo
-		$self->_sendto_menu($menu, $source)
+		$self->_sendto_menu($menu, $link)
 			if $self->{sendto_dir};
 		
 		# Open with...
-		Gtk2::Ex::DesktopEntryMenu->populate_menu($menu, $source)
+		Gtk2::Ex::DesktopEntryMenu->populate_menu($menu, "$link")
 			if $has_mimeapplications;
 	}
 	else {
-		# Separator
-		$seperator = Gtk2::MenuItem->new();
-		$seperator->show;
-		$menu->prepend($seperator);
-	
+		_separator($menu);
+
 		# Open link
 		my $open_item = Gtk2::MenuItem->new(
 			__('_Open link') ); #. context menu
@@ -1147,25 +1231,32 @@ sub on_populate_popup {
 	
 }
 
+sub _separator {
+	my $menu = shift;
+	my $item = Gtk2::SeparatorMenuItem->new();
+	$item->show;
+	$menu->prepend($item);
+}
+
 sub _sendto_menu {
 	my ($self, $menu, $file) = @_;
-	$file = Zim::File->abs_path($file);
+	$file = Zim::FS->abs_path($file);
 
 	my $st_dir = $self->{sendto_dir} || return;
 	my @dirs = ($st_dir);
 	
-	if (my $mt = mimetype($file)) {
+	if (my $mt = mimetype("$file")) {
 		$mt =~ m#(.+?)/(.+)#;
 		push @dirs, "$st_dir/.$1" if -d "$st_dir/.$1";
 		push @dirs, "$st_dir/.$1_$2" if -d "$st_dir/.$1_$2";
 	}
 	
 	my $st_menu = Gtk2::Menu->new();
-	for my $dir (reverse @dirs) {
-		for my $script (Zim::File->list_dir($dir)) {
+	for my $dir (dir($_), reverse @dirs) {
+		for my $script ($dir->list) {
 			my $item = Gtk2::MenuItem->new_with_label($script);
-			$item->signal_connect(
-				activate => sub {Zim::Utils->run("$dir/$script", $file)} );
+			$item->signal_connect(activate =>
+				sub {Zim::Utils->run($dir->file($script), $file)} );
 			$st_menu->append($item);
 		}
 	}
@@ -1189,7 +1280,7 @@ sub on_key_press_event { # some extra keybindings
 	my ($htext, $event, $self) = @_;
 	my $val = $event->keyval;
 
-	if ($self->{read_only}) {
+	if ($$self{app}{read_only}) {
 		# Unix like key bindings for read-only mode
 		if ($val == ord '/') {
 			$self->Find;
@@ -1246,11 +1337,12 @@ sub on_key_press_event { # some extra keybindings
 			return 1;
 		}
 	}
-	elsif ($val == ord('*') or $val == $k_multiply) { # Bullet
+	elsif ($val == ord('*') or $val == $k_multiply or $val == ord('>')) { # Bullet or Quote
 		my $buffer = $htext->get_buffer;
 		my ($start, $end) = $buffer->get_selection_bounds;
 		return 0 if !$end or $end == $start;
-		$self->toggle_bullets($start, $end);
+		my $char = ($val == ord('>')) ? '>' : '' ;
+		$self->toggle_bullets($start, $end, $char);
 		return 1;
 	}
 	elsif ($val == $k_home and not $event->state >= 'control-mask') { # Home toggle
@@ -1395,32 +1487,10 @@ sub load_page_contents {
 	$self->{buffer}->clear if $self->{buffer};
 	$self->{app}->Spell->detach($self->{htext}) if $use_spell;
 	$self->{_prev_buffer} = $self->{buffer}; # FIXME hack to prevent segfaults
-	
-	# create a new HyperTextBuffer
-	my $buffer = Gtk2::Ex::HyperTextBuffer->new();
-	$buffer->create_default_tags;
-#	$buffer->create_tag('info', editable => 0); #, foreground => 'red');
-	$self->{buffer} = $buffer;
-	$self->{htext}->set_buffer($buffer);
+	# set up the new buffer
+	$self->_init_buffer;
+	my $buffer = $$self{buffer};
 	$buffer->set_parse_tree( $tree );
-	$self->{read_only} = 
-		$self->{app}{read_only} ||
-		$self->{app}{page}{properties}{read_only};
-	if ($self->{read_only}) {
-		$self->{htext}->set_editable(0);
-		$self->{actions}->set_sensitive(0) if defined $self->{actions};
-	}
-	else {
-		$self->{htext}->set_editable(1);
-		$self->{actions}->set_sensitive(1);
-
-		# connect these signals _after_ load_parsetree()
-		$buffer->signal_connect(modified_changed =>
-			sub {$self->{app}->update_status} );
-		$buffer->signal_connect(edit_mode_changed => \&on_edit_mode_changed, $self);
-	}
-	$self->on_show_cursor;
-	$buffer->{_zim_name} = $self->{app}{name}; # FIXME FIXME FIXME
 	$self->{app}->Spell->attach($self->{htext}) if $use_spell;
 	
 	if ($page->exists) {
@@ -1431,8 +1501,7 @@ sub load_page_contents {
 		my (undef, $iter) = $buffer->get_bounds;
 		$buffer->place_cursor( $iter );
 	}
-	on_edit_mode_changed($buffer, $self)
-		unless $self->{read_only}; # reset format buttons
+	on_edit_mode_changed($buffer, $self);
 	
 	$self->{htext}->scroll_mark_onscreen( $buffer->get_insert );
 
@@ -1566,7 +1635,9 @@ sub parse_line {
 		grep {$_ eq $Verbatim}
 		$buffer->get_edit_mode_tags() ); # reset all tags except Verbatim
 	my $lf = $buffer->get_iter_at_line( $iter->get_line );
-	my $line = $buffer->get_text($lf, $iter, 0);
+	my $line = $buffer->get_slice($lf, $iter, 0);
+		# Need to use get_slice instead of get_text to avoid deleting
+		# images because we think a range is empty
 	#print ">>$line<<\n";
 	if ($line =~ s/^(=+)\s*(\w)/$2/) { # heading
 		my $offset;
@@ -1587,6 +1658,7 @@ sub parse_line {
 		return 1;
 	}
 	elsif ($line =~ /^(\s*(:?\W+|\d+\W?|\w\W)\s+|\s+)$/) { # empty bullet or list item
+		return 0 if $line =~ /\x{fffc}/; # embedded image
 		# TODO check previous line for same pattern !
 		_user_action_ { $buffer->delete($lf, $iter) } $buffer;
 	}
@@ -1727,22 +1799,21 @@ sub parse_word {
 	return 0;
 }
 
-my %_entities;
+my %_entities; # cache entity lookup
 
 sub _entity {
 	my $key = shift;
 	return chr($key) if $key =~ /^\d+$/;
-	my $ord;
-	if (exists $_entities{$key}) { return $_entities{$key} }
-	else {
-		for (xdg_data_files(qw/zim entities.list/)) {
-			$ord = Zim::File->new($_)->read_hash($key, 1);
-			last if defined $ord;
+	unless (exists $_entities{$key}) {
+		for (data_files(qw/zim entities.list/)) {
+			for (file($_)->read) {
+				next unless /^\Q$key\E\s+(\d+)/;
+				$_entities{$key} = chr($1);
+				last;
+			}
 		}
 	}
-	return unless length $ord;
-	$ord =~ /(\d+)/; # ignore comments
-	return chr($1);
+	return $_entities{$key};
 }
 
 sub _match_words {
@@ -1803,17 +1874,19 @@ bullets.
 =cut
 
 sub toggle_bullets {
-	my ($self, $start, $end) = @_;
+	my ($self, $start, $end, $char) = @_;
 	my $buffer = $self->{buffer};
 	($start, $end) = $buffer->get_selection_bounds unless defined $start;
 	return if !$end or $start == $end;
 	
 	my $text = $self->{buffer}->get_text($start, $end, 1);
-	if ($text =~ /^\s*[\*\x{2022}]\s+/m) { # remove bullets
-		$text =~ s/^(\s*)[\*\x{2022}]\s+/$1/mg
+	my $match = length($char) ? quotemeta($char) : qr/[\*\x{2022}]/;
+	if ($text =~ /^\s*$match\s+/m) { # remove bullets
+		$text =~ s/^(\s*)$match\s+/$1/mg
 	}
 	else { # set bullets
-		$text =~ s/^(\s*)(\S)/$1\x{2022} $2/mg;
+		$char ||= "\x{2022}";
+		$text =~ s/^(\s*)(\S)/$1$char $2/mg;
 	}
 
 	_user_action_ { $buffer->replace_selection($text) } $buffer;
@@ -1887,14 +1960,15 @@ sub selection_backspace {
 }
 
 sub _prompt_link_properties {
-	my ($self, $link, $text, $edit) = @_;
+	my ($self, $link, $text, $edit, $ext) = @_;
 
 	# Run the dialog
 	my $val = $self->run_prompt(
 		($edit ? __('Edit Link') : __('Insert Link')), #. dialog title
 		['txt', 'link'], {
 			txt  => [__('Text'), 'string', $text], #. dialog label
-			link => [__('Links to'), 'page', $link], #. dialog label
+			link => [__('Links to'), #. dialog label
+				($ext ? 'file' : 'page'), $link],
 		}, 'gtk-connect', __('_Link') ) #. dialog button
 		or return;
 	($text, $link) = @$val;
@@ -1907,16 +1981,26 @@ sub _prompt_link_properties {
 	return $link, $text;
 }
 
-sub _prompt_image_properties {
-	my ($self, $pixbuf, %meta) = @_;
+# This is a quite complicated dialog because we want to have the
+# properties for resizing the image under an expander. So we have
+# some code to add the expander and add a second form under it.
+# Also the logic for resizing forces the width and height to stay
+# locked to each other in order to keep a fixed ration. When we
+# return we only return either width or height, depending on which
+# one the user changed last.
 
-	my $file = $meta{src} || ($pixbuf ? $pixbuf->{image_src} : '');
-	my $dir = $self->{app}{page}->properties->{base};
+sub _prompt_image_properties {	
+	my ($self, $pixbuf, %meta) = @_;
+	%meta = (%{$$pixbuf{image_data}}, %meta)
+		if $pixbuf and $$pixbuf{image_data};
+
+	# Base dialog with input field for file name
 	my ($dialog, $entries) = $self->new_prompt(
 		($pixbuf ? __('Edit Image') : __('Insert Image') ), #. dialog title
-		['src'], {src => [__('Source'), 'file', $file, $dir]}, #. dialog label
+		['src'], {src => [__('Source'), 'file', $meta{src}]}, #. dialog label
 		'gtk-connect', __('_Link') ); #. dialog button
 	
+	# Add expander with advanced properties
 	my ($w, $h, $ratio) = (0, 0, 1);
 	if ($pixbuf) {
 		($w, $h) = ($pixbuf->get_width, $pixbuf->get_height);
@@ -1926,99 +2010,125 @@ sub _prompt_image_properties {
 	my $frame = Gtk2::Expander->new_with_mnemonic(
 		__('_Properties') ); #. expander label
 	$dialog->vbox->add($frame);
-	my $vbox = Gtk2::VBox->new();
-	$vbox->set_border_width(12);
-	$frame->add($vbox);
+	# logic to resize dialog after hiding the exander..
+	# listening to notify::expanded doesn't work since the size did not
+	# change yet when that event is emitted.
+	$frame->signal_connect_after(size_request => sub {
+		return if $frame->get_expanded;
+		my ($width, $height) = $dialog->get_size;
+		$dialog->resize($width, 1);
+			# do not touch width, minimize height
+	} );
 
-	my $table = Gtk2::Table->new(2, 2, 0);
-	$table->set_row_spacings(5);
-	$table->set_col_spacings(12);
-	$vbox->add($table);
+	# Create the form for inside the expander. Allow size to range
+	# from 1 pixel to 4 times the current size (yes, this is arbitrary).
+	# Step size will be 1 of course.
+	my ($table, $spin) = $self->new_form(
+		['width', 'height'], {
+			width  => [__('Width'),  'int', $w, [1, 4*$w||1, 1]], #. input label
+			height => [__('Height'), 'int', $h, [1, 4*$h||1, 1]], #. input label
+		} );
+	$table->set_border_width(24);
+	$frame->add($table);
 
-	my $i = 0;
-	for (__('Width'), __('Height')) { #. label
-		my $label = Gtk2::Label->new($_.':');
-		my $align = Gtk2::Alignment->new(0,0.5, 0,0);
-		$align->add($label);
-		$table->attach_defaults($align, 0,1, $i,$i+1);
-		$i++;
-	}
-
-	my $input_w = Gtk2::SpinButton->new_with_range(1, 4*$w || 1, 1);
-	$input_w->set_value($w);
-	$table->attach_defaults($input_w, 1,2, 0,1);
-	my $input_h = Gtk2::SpinButton->new_with_range(1, 4*$h || 1, 1);
-	$input_h->set_value($h);
-	$table->attach_defaults($input_h, 1,2, 1,2);
-
-	my $reset_button = Gtk2::Button->new(__('_Reset')); #. button
-	$vbox->pack_start($reset_button, 0,0,0);
-
-	my ($scale, $block) = ('', 0);
-	if ($pixbuf and $pixbuf->{resized}) {
-		$scale = $pixbuf->{resized}[0] ? 'w' : 'h';
-	}
-	$input_w->signal_connect(value_changed => sub {
+	my $block = 0;
+		# only react to user changes, not internal ones
+	my $changed = $meta{width} ? 'w' : $meta{height} ? 'h' : undef ;
+		# use this parameter to keep track of latest change
+	$$spin[0]->signal_connect(value_changed => sub {
+		# width changed, update height according to ratio
 		return if $block;
-		$scale = 'w';
-		$w = $input_w->get_value;
+
+		$changed = 'w';
+		$w = $$spin[0]->get_value;
 		$h = int $w / $ratio;
+
 		$block = 1;
-		$input_h->set_value($h);
+		$$spin[1]->set_value($h);
 		$block = 0;
 	} );
-	$input_h->signal_connect(value_changed => sub {
+	$$spin[1]->signal_connect(value_changed => sub {
+		# heigth changed, update width according to ratio
 		return if $block;
-		$scale = 'h';
-		$h = $input_h->get_value;
+
+		$changed = 'h';
+		$h = $$spin[1]->get_value;
 		$w = int $ratio * $h;
+
 		$block = 1;
-		$input_w->set_value($w);
+		$$spin[0]->set_value($w);
 		$block = 0;
 	} );
-	my $init = sub {
+
+	# this button resets height and width to the actual image size
+	my $reset_button = Gtk2::Button->new(__('_Reset')); #. button
+	$table->attach_defaults($reset_button, 1,2, 2,3);
+
+	$reset_button->signal_connect(clicked => sub {
+		# This will read the dimensions from the actual file and
+		# set the spin buttons.
+		$_->set_sensitive(0) for @$spin;
+		$changed = undef;
+
 		my $file = $$entries[0]->get_text;
-		$file = File::Spec->rel2abs($file, $self->{buffer}{base})
-			if $self->{buffer}{base};
+		return unless length $file;
+		$file = $self->check_file_input($file);
+		
+		($w, $h) = (0, 0);
 		eval { # dies on my win32 installation
-			(undef, $w, $h) = (-f $file and -r _)
-				? (Gtk2::Gdk::Pixbuf->get_file_info($file))
-				: (0,0) ;
-		};
+			(undef, $w, $h) =
+				Gtk2::Gdk::Pixbuf->get_file_info($file);
+		} if -f $file and -r _ ;
+
 		if ($w && $h) {
-			$_->set_sensitive(1)
-				for $reset_button, $input_w, $input_h;
+			# file exists and eval did not die
 			$ratio = $w / $h;
-			$block = 1;
-			$input_w->set_range(1, 4*$w);
-			$input_w->set_value($w);
-			$input_h->set_range(1, 4*$h);
-			$input_h->set_value($h);
+			$block = 1; # block signals
+			$$spin[0]->set_range(1, 4*$w);
+			$$spin[0]->set_value($w);
+			$$spin[1]->set_range(1, 4*$h);
+			$$spin[1]->set_value($h);
 			$block = 0;
-			$scale = '';
+			$_->set_sensitive(1) for @$spin;
 		}
 		else {
-			$_->set_sensitive(0)
-				for $reset_button, $input_w, $input_h;
+			# image does not exist or get_info died
+			# anyway spin buttons will be insensitive
+			$_->set_value(0) for @$spin;
 		}
-	};
-	$reset_button->signal_connect(clicked => $init);
-	$$entries[0]->signal_connect(changed => $init);
-	$init->() unless $pixbuf;
+	} );
+	$reset_button->clicked unless $pixbuf; # init for new files
 
+	# Run the dialog
 	$dialog->show_all;
 	my $r = $dialog->run;
 	$dialog->destroy;
 	return unless $r eq 'ok';
 
-	my $new_src = $$entries[0]->get_text;
-	return unless $new_src =~ /\S/;
+	# get the file name and optionally the properties
+	my $path = $$entries[0]->get_text;
+	return unless $path =~ /\S/;
+
+	my $file = $self->check_file_input($path);
+	$meta{file} = $file->path;
+	$meta{src} = $$self{app}{page}->relative_path($file);
+		# The absolute path, $meta{file} is used to create the
+		# pixbuf etc. The human readable $meta{src} is used to
+		# present to the user and is the path that will be saved
+		# in the wiki source.
+	if ($changed eq 'w') { # width was set explicitly
+		$meta{width}  = $w;
+		delete $meta{height};
+	}
+	elsif ($changed eq 'h') { # heigth was set explicitly
+		$meta{height} = $h;
+		delete $meta{width};
+	}
+	else { # we used reset
+		delete $meta{width};
+		delete $meta{height};
+	}
 
-	$meta{src} =
-		$self->{app}{page}->relative_path(
-			Zim::File->abs_path($new_src, $dir) );
-	@meta{'width', 'height'} = ($scale eq 'w') ? ($w, 0) : (0, $h)
-		if $scale;
 	return %meta;
 }
 
