@@ -4,8 +4,9 @@ use strict;
 use vars '$AUTOLOAD';
 use Gtk2;
 use Gtk2::Ex::PathBar; # custom widget
+use Zim::Utils;
 
-our $VERSION = '0.19';
+our $VERSION = '0.23';
 
 =head1 NAME
 
@@ -124,7 +125,8 @@ sub _load_namespace {
 	for ($self->{path_bar}->get_items()) {
 		my $name = ':'.join(':', @{$_->{path_data}}) ;
 		$_->signal_connect(enter => sub {
-			$self->{app}->push_status("Go to \"$name\"", 'link') } );
+			$self->{app}->push_status(
+				__("Go to \"{link}\"", link => $name), 'link') } ); #. status bar info
 		$_->signal_connect(leave => sub {
 			$self->{app}->pop_status('link') } );
 	}
@@ -157,7 +159,8 @@ sub _load_history {
 	for ($self->{path_bar}->get_items) {
 		my $name = shift @names;
 		$_->signal_connect(enter => sub {
-			$self->{app}->push_status("Go to \"$name\"", 'link') } );
+			$self->{app}->push_status(
+				__("Go to \"{link}\"", link => $name), 'link') } ); #. status bar info
 		$_->signal_connect(leave => sub {
 			$self->{app}->pop_status('link') } );
 	}
@@ -188,7 +191,8 @@ sub _load_recent {
 	for ($self->{path_bar}->get_items) {
 		my (undef, $name) = @{shift @path};
 		$_->signal_connect(enter => sub {
-			$self->{app}->push_status("Go to \"$name\"", 'link') } );
+			$self->{app}->push_status(
+				__("Go to \"{link}\"", link => $name), 'link') } ); #. status bar info
 		$_->signal_connect(leave => sub {
 			$self->{app}->pop_status('link') } );
 	}
