@@ -6,7 +6,7 @@ use Gtk2::Spell;
 use Zim::Utils;
 use Zim::GUI::Component;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 our @ISA = qw/Zim::GUI::Component/;
 
@@ -63,7 +63,8 @@ sub init {
 	$self->add_actions($ui_toggle_actions, 'TOGGLE');
 	$self->add_ui($ui_layout);
 	$self->actions_set_active(
-		TSpell => $self->{app}{settings}{show_spell} );
+		TSpell => $self->{app}{state}{show_spell} );
+	$self->init_settings('Spell Plugin', {Language => ''});
 }
 
 =item C<TSpell()>
@@ -76,7 +77,7 @@ sub TSpell {
 	my $self = shift;
 	$self->{show} = $self->{show} ? 0 : 1;
 	$self->{show} ? $self->attach : $self->detach;
-	$self->{app}{settings}{show_spell} = $self->{show};
+	$self->{app}{state}{show_spell} = $self->{show};
 	#$self->actions_show_active(TSpell => $self->{show});
 }
 
@@ -99,7 +100,7 @@ sub attach {
 	return unless $textview and $self->{show};
 
 	eval {
-		my $lang = $self->{app}{settings}{spell_language};
+		my $lang = $self->{settings}{Language};
 		$self->{object} = Gtk2::Spell->new_attach($textview);
 		$self->{object}->set_language($lang) if length $lang;
 		$self->{object}->recheck_all;
