@@ -1,7 +1,7 @@
 package Zim::History;
 
 use strict;
-use Zim::File;
+use Zim::Utils;
 use Zim::Page;
 
 our $VERSION = '0.20';
@@ -52,7 +52,7 @@ called on this object to get state information that needs to be saved.
 
 sub new {
 	my ($class, $file, $max, $gui) = @_;
-	$file = Zim::File->new($file) if defined $file and ! ref $file;
+	$file = file($file) if defined $file;
 	my $self = bless {
 		file   => $file,
 		max    => $max,
