@@ -5,9 +5,10 @@ use overload
 	'""' => sub { $_[0]->{name} },
 	fallback => 'TRUE' ;
 use Carp;
+use Zim::Utils;
 use Zim::Formats;
 
-our $VERSION = '0.22';
+our $VERSION = '0.24';
 
 our %_Formats;
 
@@ -45,7 +46,7 @@ sub new {
 	my $self = bless {
 		name	=> $name,
 		store	=> $parent,
-		root	=> $parent ? $parent->root : undef,
+		root	=> ($parent ? $parent->root : undef),
 		exists	=> 1, # only used when no source set
 		status	=> '',
 		properties => {read_only => 1},
@@ -165,7 +166,7 @@ sub move {
 
 sub delete { $_[0]->{store}->delete_page(@_) }
 
-=item C<parse_link(LINK)>
+=item C<parse_link(LINK, NO_RESOLVE)>
 
 Returns a link type and a link target, see L<Zim::Formats>.
 
@@ -173,9 +174,11 @@ Returns a link type and a link target, see L<Zim::Formats>.
 
 sub parse_link {
 	my $self = shift;
-	return $self->{cloning}->parse_link(@_) if $self->{cloning};
+	# No cloning hack here, we only need to resolve pages from the
+	# notebook we are cloning, not files, interwiki etc.
+	# So cloning is take care of in resolve_name
 	my $fmt = $self->{format} || 'Zim::Formats';
-	return $fmt->parse_link(@_, $self);
+	return $fmt->parse_link(shift, $self, @_);
 }
 
 =item C<resolve_page(NAME, NO_DEFAULT)>
@@ -194,11 +197,14 @@ sub resolve_page {
 	return $name ? $self->{root}->get_page($name) : undef ;
 }
 
-sub resolve_name { # 'cloning' hack needed when exporting
+sub resolve_name {
 	my ($self, $link, $no_def) = @_;
 	return $self->{cloning}->resolve_name($link, $no_def)
 		if $self->{cloning};
-
+		# 'cloning' hack needed when exporting because we may need
+		# to resolve pages that do not yet exist in our new notebook.
+		# So we use the notebook we are cloning from.
+	
 	if (my $r = $self->{_resolve}{$link}) {
 		# Caching
 		return ( ($no_def && $$r[1]) ? undef : $$r[0] )
@@ -243,11 +249,19 @@ sub equals {
 	return $self->{name} eq $page->name;
 }
 
-=item C<clone(SOURCE)>
+=item C<clone(PAGE, media => MEDIA, doc_root => PREFIX)>
 
-Import content from object SOURCE into this object.
+Import content from page object PAGE into this object.
+PAGE typically belongs to a different notebook.
 Deletes current contents.
 
+MEDIA can be 'none', 'default' or 'all'. When 'none' is set no media is copied.
+When 'default' is set all files below the document dir root are copied.
+For 'all' we automatically attach external files.
+
+PREFIX is a prefix we use for links under the document root. If this is set
+any files under the document root will not be copied for any setting of MEDIA.
+
 =cut
 
 sub clone {
@@ -256,61 +270,86 @@ sub clone {
 	if ( $self->{format} and $page->{format} ) {
 		$self->delete if $self->exists; # prevent mtime check etc.
 		my $tree = $page->get_parse_tree;
-		$self->_check_media($tree, $opt{media}, $page)
-			if $opt{media} and $opt{media} ne 'relative';
+		$self->_check_media($tree, $page, %opt);
 		$self->set_parse_tree($tree);
 	}
 	else { warn "Could not clone page: ". $page->name ."\n" }
 }
 
+# Check for all file links and image in the parse tree if we need
+# to copy the corresponding files to the exported file tree
+
 sub _check_media {
-	my ($self, $tree, $media, $cloning) = @_;
-	#warn "_check_media: @_\n";
-	die "BUG: unknown media option: $media"
-		unless $media eq 'absolute' or $media eq 'copy';
+	my ($self, $tree, $cloning, %opt) = @_;
+	$opt{media} ||= 'default';
+	$opt{doc_root} = $$self{store}->document_root;
+	die "BUG: unknown media option: $opt{media}"
+		unless grep {$opt{media} eq $_} qw/none default all/;
 	
-	my $base = $cloning->{properties}{base};
 	for my $l (Zim::Formats->extract_refs('link', $tree)) {
-		my ($type, $link) = $self->parse_link($$l[1]{to});
+		my ($type, $link) = $cloning->parse_link($$l[1]{to});
 		next unless $type eq 'file';
-		$link = Zim::File->abs_path($link, $base);
-		$$l[1]{to} = ($media eq 'absolute')
-			? $link : $self->_copy_media($link, $cloning);
+		$$l[1]{to} = $self->_copy_media($link, $cloning, %opt);
 	}
 
 	for my $i (Zim::Formats->extract_refs('image', $tree)) {
-		my $link = $$i[1]{src};
-		$link = Zim::File->abs_path($link, $base);
-		$$i[1]{src} = ($media eq 'absolute')
-			? $link : $self->_copy_media($link, $cloning);
+		my $link = delete $$i[1]{file};
+			# for images 'file' gives fully resolved path
+			# need to set 'src' because 'file' is not saved
+			# but do delete old path in 'file'
+		$$i[1]{src} = $self->_copy_media($link, $cloning, %opt);
 	}
 }
 
 sub _copy_media {
-	my ($self, $file, $cloning) = @_;
+	my ($self, $file, $cloning, %opt) = @_;
+		# Cloning is the page we are cloning _from_
 
+	return $self->relative_path($file) if $opt{media} eq 'none';
+		# We use relative path here instead of absolute
+		# because maybe we are using clone to copy within the
+		# same notebook between different storage backends
+
+	my $path = $cloning->relative_path($file);
+		# returns e.g. ./file, /file or ~/file
+		# if cloning has a doc_root /file is below doc_root and
+		# absolute paths will start with file://
+		# If we have a doc_root use it as is, later parse_link can
+		# prefix our doc_root to is, if we do not have a doc_root
+		# we use the original absolute path to attach or link.
+
+	# Determine if we need to copy, and if so to where
 	my $copy;
-	my $croot = $cloning->{root}{dir};
-	my $root = $self->{root}{dir};
-	my $rel = Zim::File->rel_path($file, $croot);
-	if (defined $rel) { # was relative
-		$copy = Zim::File->abs_path($rel, $root);
+	if ($path =~ /^\./) {
+		# file is relative to document dir - copy
+		$copy = $self->resolve_file($path);
 	}
-	else { # was absolute
-		my $k = $file;
-		$k =~ s#[/\\]+#_#g; # replace dir separater
-		$k =~ s#^_+##;
-		$copy = Zim::File->abs_path($k, $root.'/_media');
+	elsif (defined $opt{doc_root} and $path =~ /^\//) {
+		# link to a file inside document root - use as is
+		return $path;
+	}
+	elsif ($opt{media} eq 'all') {
+		# link outside doc_dir and doc_root (if we have any)
+		# copy to _media directory
+		my $key = $file;
+		$key =~ s/[\/\\]+/_/g; # remove dir separators
+		$key =~ s/^_+|_+$//g;
+		$copy = $$self{store}->document_dir()->file("_media/$key");
+	}
+	else {
+		# Return absolute path - use uri to force abs path
+		return $opt{doc_root} ? Zim::FS->path2uri($file) : $file ;
 	}
 
+	# Do the actual copy
+	# TODO: we can skip the actual copy if the file was already copied
+	# for another page linking the same file. However checking existence 
+	# is not enough: maybe we are overwriting a previous export.
 	if (-f $file) {
 		warn "#  copy $file => $copy\n";
-		Zim::File->copy($file => $copy);
+		file($file)->copy($copy);
 	}
-
-	my $link = Zim::File->rel_path(
-		$copy, $self->{properties}{base}, 1);
-	return $link || $copy ;
+	return $self->relative_path($copy);
 }
 
 =item C<get_next()>
@@ -396,7 +435,7 @@ sub open_source {
 	my $src = $self->{source};
 	return unless defined $src;
 	return if $mode eq 'r' and ! $src->exists;
-	$src->make_dir if $mode eq 'w' and $src->can('make_dir');
+	$src->dir->touch if $mode eq 'w' and $src->can('dir');
 	my $fh = $src->open($mode);
 	$self->{status} = '' if $mode eq 'w' and defined $fh;
 		# remove "new" or "deleted" status
@@ -492,9 +531,7 @@ sub set_parse_tree { #warn "set_parse_tree from ", join(' ', caller), "\n" ;
 	if (defined $self->{source}) {
 		my $io = $self->open_source('w')
 			|| die "Could not save parse tree, did not get an IO object.\n";
-		#flock($io, LOCK_EX); # lock # Move function to IO::File::Zim
 		$self->{format}->save_tree($io, $tree, $self);
-		#flock($io, LOCK_UN); # unlock
 		$io->close;
 		
 		$self->{store}->_cache_page($self)
@@ -528,7 +565,6 @@ sub list_links { #warn "list_links from ", join(' ', caller), "\n" ;
 			my ($type, $link) = ('', $$n[1]{to});
 			($type, $link) = $self->parse_link($link);
 			next unless $type eq 'page';
-			$link = $self->resolve_name($link);
 			$links{$link} = 1;
 		}
 		else {
@@ -545,8 +581,11 @@ Returns a list with names of pages that link to this page.
 
 =cut
 
-sub list_backlinks { # 'cloning' hack needed when exporting
+sub list_backlinks {
 	my $obj = $_[0]->{cloning} ? $_[0]->{cloning} : $_[0];
+		# 'cloning' hack needed when exporting because we may need
+		# to resolve pages that do not yet exist in our new notebook.
+		# So we use the notebook we are cloning from.
 	return $obj->{root}->can('list_backlinks')
 		? $obj->{root}->list_backlinks($obj) : () ;
 }
@@ -573,12 +612,13 @@ sub update_links {
 	my $done = 0;
 	for my $ref (Zim::Formats->extract_refs('link', $tree)) {
 		my $old = $$ref[1]{to};
-		my ($t, $l) = $self->parse_link($old);
+		my ($t, $l) = $self->parse_link($old, 'NO_RESOLVE');
 		#warn "found link, old = $old => $t, $l\n";
 		next unless $t eq 'page';
-		my $match = $selection->resolve_name($l, $self, 1) || next;
+		my $match = $selection->resolve_name($l, $self, 'NO_DEFAULT')
+			|| next;
 		#warn "\told match = $match\n";
-		my $new_match = $self->resolve_name($l, 1);
+		my $new_match = $self->resolve_name($l, 'NO_DEFAULT');
 		#warn "\tnew match = $new_match\n";
 		next if $new_match and length($new_match) > length($match);
 			# the only difference is the movement of pages
@@ -608,7 +648,7 @@ sub update_links_self {
 	my $done = 0;
 	for my $ref (Zim::Formats->extract_refs('link', $tree)) {
 		my $old = $$ref[1]{to};
-		my ($t, $l) = $self->parse_link($old);
+		my ($t, $l) = $self->parse_link($old, 'NO_RESOLVE');
 		#warn "found link, old = $old => $t, $l\n";
 		next unless $t eq 'page';
 		my $page = $self->{root}->resolve_name($l, $from);
@@ -630,6 +670,40 @@ sub update_links_self {
 
 =back
 
+=head2 Document Interface
+
+These methods are used to store and retrieve documents. This has nothing to
+do with the source of the page, these documents should be regarded as
+attachments.
+
+See detailed documentation in L<Zim::Store>.
+
+=over 4
+
+=item C<document_dir()>
+
+=item C<store_file(FILE, NAME, MOVE)>
+
+=item C<resolve_file(PATH)>
+
+=item C<relative_path(FILE)>
+
+=cut
+
+sub document_dir { $_[0]{store}->document_dir($_[0]) }
+
+sub store_file {
+	my ($self, $file) = (shift, shift);
+	croak "BUG: '$file' is not an object" unless ref $file;
+	$$self{store}->store_file($file, $self, @_);
+}
+
+sub resolve_file { $_[0]{store}->resolve_file($_[1], $_[0]) }
+
+sub relative_path { $_[0]{store}->relative_path($_[1], $_[0]) }
+
+=back
+
 =head2 Other Functions
 
 =over 4
@@ -657,27 +731,6 @@ sub relative_name {
 }
 
 
-=item C<relative_path(FILE, PAGE)>
-
-Checks to see if the path for FILE can be made relative
-to the base dir for PAGE.
-Returns either the relative or the absolute path.
-
-=cut
-
-sub relative_path {
-	my ($self, $file) = @_;
-	return $file if $file =~ m#^\w[\w\+\-\.]+:/#; # url
-
-	my $root = $self->{root}{dir};
-	my $rel = Zim::File->rel_path($file, $root, 0);
-	return $file unless defined $rel; # file not below root
-
-	my $ref = $self->{properties}{base};
-	$rel = Zim::File->rel_path($file, $ref, 1);
-	return defined($rel) ? $rel : $file ;
-}
-
 =item C<get_text()>
 
 Returns plain text page. Used by "Email Page" action.
@@ -689,12 +742,11 @@ sub get_text {
 	my $self = shift;
 	my $tree = $self->get_parse_tree;
 	my $fmt = 'Zim::Formats::Wiki';
-	my ($io, $text);
-	open $io, '>', \$text;
-	binmode $io, ':utf8' unless $] < 5.008;
-	$fmt->save_tree($io, $tree, $self);
-	close $io;
-	return $text;
+	my $buffer = buffer();
+	my $fh = $buffer->open('w');
+	$fmt->save_tree($fh, $tree, $self);
+	close $fh;
+	return ''.$buffer->read;
 }
 
 =item C<commit_change()>
@@ -703,9 +755,15 @@ sub get_text {
 
 =cut
 
-sub commit_change { $_[0]->{source}->commit_change if $_[0]->{source} }
+sub commit_change {
+	$_[0]{source}->commit_change
+		if $_[0]{source} and $_[0]{source}->can('commit_change');
+}
 
-sub discard_change { $_[0]->{source}->discard_change if $_[0]->{source} }
+sub discard_change {
+	$_[0]{source}->discard_change
+		if $_[0]{source} and $_[0]{source}->can('discard_change');
+}
 
 1;
 
