@@ -1,8 +1,7 @@
 use strict;
 use File::Spec;
-use Zim::File;
-use Zim::Page;
-use Zim::Repository::Files;
+use Zim::Utils;
+use Zim::Store::Files;
 
 =head1 NAME
 
@@ -10,7 +9,7 @@ PrintToBrowser
 
 =head1 VERSION
 
-0.18
+0.24
 
 =head1 DESCRIPTION
 
@@ -27,19 +26,15 @@ modify it under the same terms as Perl itself.
 
 =cut
 
-my $app  = Zim::GUI->current;
+my $app = Zim::GUI->current;
 
-$app->{actions}->add_actions( [
-	[	'TmpPrint',
-		'gtk-print',
-		'Print to Browser',
-		'<Ctrl>P',
-		'Print to browser',
-		\&on_TmpPrint,
-	],
-] );
+my $actions = __actions( q{
+TmpPrint	gtk-print	Print to Browser	<Ctrl>P	Print to browser
+} );
 
-$app->{ui}->add_ui_from_string( q{
+$app->add_actions($actions);
+
+$app->add_ui( q{
 <ui>
 	<menubar name='MenuBar'>
 		<menu action='FileMenu'>
@@ -50,30 +45,29 @@ $app->{ui}->add_ui_from_string( q{
 	</menubar>
 </ui> } );
 
-sub on_TmpPrint {
+return 1;
+
+
+sub TmpPrint {
 	warn "No real printing, tmp solution\n";
-	my $tmpdir = File::Spec->tmpdir;
+	eval "use Zim::Template"; die $@ if $@;
 
 	# Export page to html
-	my $file = "$tmpdir/zim-PrintToBrowser-$ENV{USER}.html";
-	my $mask = umask 0077; # set umask to user only
+	my $file = Zim::FS->tmp_file("PrintToBrowser.html");
 
-	my $exporter = Zim::Repository::Files->new(dir => $tmpdir);
+	my $exporter = Zim::Store::Files->new(dir => File::Spec->tmpdir);
 	my $template = Zim::Formats->lookup_template('html', 'Print');
 	return $app->error_dialog("Could not find template for PrintToBrowser")
 		unless defined $template;
-	$exporter->{_template} =  $template;
-	my ($rep, $page) = @{$app}{'repository', 'page'};
+	$exporter->{_template} = Zim::Template->new($template);
+	#warn "TEMPLATE: $$exporter{_template}\n";
+	my ($rep, $page) = @{$app}{'notebook', 'page'};
 	my $tmp = Zim::Page->new($exporter, $page->name);
-	$tmp->set_source( Zim::File->new($file) );
+	$tmp->set_source($file);
 	$tmp->set_format('html');
-	$tmp->properties->{base} = $page->properties->{base};
-	$tmp->clone($page, media => 'absolute');
-	warn "Exported to $file\n";
-
-	umask $mask; # reset old umask
+	$tmp->clone($page, media => 'none');
+	warn "# Exported to $file\n";
 
 	# Exec Browser
-	my $uri = Zim::File->path2uri($file);
-	$app->open_url($uri);
+	$app->open_url($file->uri);
 }
