@@ -1,3 +1,5 @@
+use strict;
+
 my $app = Zim::GUI->current();
 
 eval 'use Zim::GUI::Spell';
@@ -6,10 +8,8 @@ if ($@) {
 		"Failed to load Spell plugin,\n".
 		"do you have Gtk2::Spell installed ?", $@);
 	$app->unplug('Spell');
-	return 1;
 }
 
-my $spell = Zim::GUI::Spell->new(app => $app);
-$app->add_object(Spell => $spell);
+$app->Spell; # load object - HACK - more (all?) code from module should be here
 
-1;
+return 1;
