@@ -6,7 +6,7 @@ Svn support
 
 =head1 VERSION
 
-0.23
+0.24
 
 =head1 DESCRIPTION
 
@@ -108,10 +108,8 @@ sub is_recent {
 	my $local_revision = 0;
 	my $revision = 0;
 	my $page = $app->{page};
-	my $rep = $page->{store};
-	return unless $rep->can('file');
-	my $file = $rep->file( $page );
-	my $filepath = $file->path;
+	return unless $$page{source} and $$page{source}->can('path');
+	my $filepath = $$page{source}->path;
 
 	open(OUT,"svn info \"$filepath\" 2>&1|") ;
 	while (<OUT>) {
@@ -272,12 +270,18 @@ sub on_SvnCommit {
 			}
 		}
 
-		if (system ("svn ci -m $log_msg $dir &>/dev/null")) {
-			$app->error_dialog("Failed commit changes to SVN repository!");
+		# Save comment in a temp file to allow spaces and quotes inside.
+		my $comment = Zim::FS->tmp_file('SubversionComment.txt');
+		$comment->write($log_msg);
+		my $null = Zim::FS->devnull;
+	 
+		if (system ("svn ci -F \"$comment\" \"$dir\" &>$null") == 0) {
+			warn "Commit successfull.\n";
 		}
 		else {
-			warn 'Commit successfull.';
+			$app->error_dialog("Failed commit changes to SVN repository!");
 		}
+		$comment->remove;
 	}
 
 
